
/*************************************************************************

  APU R 2.0

  bigint.c

  (c) 1996, Regents of the University of California. All rights reserved. 

  Author : Ashutosh Rege (rege@cs.berkeley.edu)

*************************************************************************/
  
#include "bigint.h"

const char* BigInt::Class_name = "BigInt";

BigInt
operator + (const BigInt& a, const BigInt&b) {
  BigInt result = BigInt(*((integer*)a.frep) + *((integer*)b.frep));
  return(result);
}

BigInt
operator - (const BigInt& a, const BigInt&b) {
  BigInt result = BigInt(*((integer*)a.frep) - *((integer*)b.frep));
  return(result);
}

BigInt
operator - (const BigInt& a) {
  BigInt result = BigInt( - (*((integer*)a.frep)));
  return(result);
}

BigInt
operator * (const BigInt& a, const BigInt&b) {
  BigInt result = BigInt(*((integer*)a.frep) * *((integer*)b.frep));
  return(result);
}

BigInt
operator / (const BigInt& a, const BigInt&b) {
  BigInt result = BigInt(*((integer*)a.frep) / *((integer*)b.frep));
  return(result);
}

BigInt
operator % (const BigInt& a, const BigInt&b) {
  BigInt result = BigInt(*((integer*)a.frep) % *((integer*)b.frep));
  return(result);
}

BigInt
operator ^ (const BigInt& a, int b) {
  if (b == 0) return(BigInt(1));
  BigInt result = a; 
  for (int i=0;i<b-1;i++)
    result = result*a;
  return(result);
}

bool
operator == (const BigInt& a, const BigInt&b) {
  return(*((integer*)a.frep) == *((integer*)b.frep));
}

bool
operator != (const BigInt& a, const BigInt&b) {
  return(*((integer*)a.frep) != *((integer*)b.frep));
}

bool
operator < (const BigInt& a, const BigInt&b) {
  return(*((integer*)a.frep) < *((integer*)b.frep));
}

bool
operator <= (const BigInt& a, const BigInt&b) {
  return(*((integer*)a.frep) <= *((integer*)b.frep));
}

bool
operator > (const BigInt& a, const BigInt&b) {
  return(*((integer*)a.frep) > *((integer*)b.frep));
}

bool
operator >= (const BigInt& a, const BigInt&b) {
  return(*((integer*)a.frep) >= *((integer*)b.frep));
}

bool
is_zero(const BigInt& a) {
  return(((integer*)a.frep)->iszero());
}

int 
sign(const BigInt& a) {
  return(sign(*((integer*)a.frep)));
}
