
/*************************************************************************

  APU R 2.0

  bigrat.c

  (c) 1996, Regents of the University of California. All rights reserved. 

  Author : Ashutosh Rege (rege@cs.berkeley.edu)

*************************************************************************/
  
#include "bigrat.h"

const char* BigRat::Class_name = "BigRat";

BigRat
operator + (const BigRat& a, const BigRat&b) {
  BigRat result = BigRat(normalize(a.rational_of() + b.rational_of())); 
  return result; 
}

BigRat
operator - (const BigRat& a, const BigRat&b) {
  BigRat result = BigRat(normalize(a.rational_of() - b.rational_of())); 
  return result; 
}

BigRat
operator - (const BigRat& a) {
  BigRat result = BigRat(-a.rational_of()); 
  return(result);
}

BigRat
operator * (const BigRat& a, const BigRat&b) {
  BigRat result = BigRat(normalize(a.rational_of() * b.rational_of())); 
  return result; 
}

BigRat
operator / (const BigRat& a, const BigRat&b) {
  BigRat result = BigRat(normalize(a.rational_of() / b.rational_of())); 
  return result; 
}

BigRat
operator ^ (const BigRat& a, int b) {
  if (b == 0) return(BigRat(1));
  else return(BigRat(pow(a.rational_of(),b))); 
}

bool
operator == (const BigRat& a, const BigRat&b) {
  return(a.rational_of() == b.rational_of()); 
}

bool
operator != (const BigRat& a, const BigRat&b) {
  return(a.rational_of() != b.rational_of()); 
}

bool
operator < (const BigRat& a, const BigRat&b) {
  return(a.rational_of() < b.rational_of()); 
}

bool
operator <= (const BigRat& a, const BigRat&b) {
  return(a.rational_of() <= b.rational_of()); 
}

bool
operator > (const BigRat& a, const BigRat&b) {
  return(a.rational_of() > b.rational_of()); 
}

bool
operator >= (const BigRat& a, const BigRat&b) {
  return(a.rational_of() >= b.rational_of()); 
}

bool
is_zero(const BigRat& a) {
  return(numer_of(a).iszero()); 
}

int 
sign(const BigRat& a) {
  return(sign(a.rational_of())); 
}
