
/*************************************************************************

  APU R 2.0

  field.c

  (c) 1996, Regents of the University of California. All rights reserved. 

  Author : Ashutosh Rege (rege@cs.berkeley.edu)

*************************************************************************/
  
#include "field.h"
#include "fieldtypes.h"

const char* Field_element::class_name() const {
  switch(type) {
    case SMALLINT : return(((SmallInt*)this)->class_name());
    case INTMODP : return(((IntModp*)this)->class_name());
    case MULTI_INTMODP : return(((MultiIntModp*)this)->class_name());
    case BIGINT  : return(((BigInt*)this)->class_name());
    case FLOAT  : return(((Float*)this)->class_name());
    case BIGFLOAT  : return(((BigFloat*)this)->class_name());
    case SLP  : return(((Slp*)this)->class_name());
    default : return("Unknown type"); 
  }
}

void Field_element::print() const { 
  switch(type) {
    case INTMODP : ((IntModp*)this)->print(); break;
    case BIGINT : ((BigInt*)this)->print(); break;
    case FLOAT : ((Float*)this)->print(); break;
    default : cout << "Unknown type\n";  break;
  }
}

Field_element::Field_element(int a) { 
  type = SMALLINT;
  frep = (int *)(new int); 
  *((int*)frep) = a;
}

Field_element::Field_element(unsigned int a) { 
  type = INTMODP;
  int temp = (int)(a % IntModp::prime); 
  if (temp < 0) temp = temp + IntModp::prime; 
  frep = (void *)((unsigned int)temp); 
}

Field_element::Field_element(int a[MULTI_NUMPRIMES]) { 
  type = MULTI_INTMODP; 
  for (int i=0;i<MultiIntModp::CurrentNumPrimes;i++) { 
    int temp = (int)(a[i] % Primes.primes[i]); 
    if (temp < 0) temp = temp+Primes.primes[i]; 
    frep2[i] = temp; 
  }
}

Field_element::Field_element(integer a) {
  type = BIGINT; 
  frep = (void*)(new integer); 
  *((integer*)frep) = a; 
}

Field_element::Field_element(double a) {
  type = FLOAT; 
  frep = (void*)(new double); 
  *((double*)frep) = a; 
}

Field_element::Field_element(bigfloat a) {
  type = BIGFLOAT; 
  frep = (void*)(new bigfloat); 
  *((bigfloat*)frep) = a; 
}

ostream&
operator<< (ostream& os, const Field_element& fe) {
  switch(fe.type_of()) {
    case VAR : os << ((Variable&)fe); break;
    case SMALLINT : os << ((SmallInt&)fe).i_value(); break;
    case INTMODP : os << ((IntModp&)fe).i_value(); break;
    case MULTI_INTMODP : os << ((MultiIntModp&)fe); break;
    case BIGINT : os << ((BigInt&)fe).i_value(); break;
    case FLOAT : os << ((Float&)fe).f_value(); break;
    case SLP : os << evaluate((Slp&)fe); break;
    default : os << "Unknown type input to <<\n"; 
  }
}

Field_element
make_Field_element(int invalue, int fe_type) {
  Field_element result;
  switch (fe_type) {
    case SMALLINT : result = Field_element(invalue); break;
    case INTMODP : 
       int temp; 
       temp = invalue % (IntModp::prime);  
       if (temp < 0) {
	 temp = temp + IntModp::prime; 
       }
       result = Field_element((unsigned)temp); break;
    case MULTI_INTMODP : 
       result = MultiIntModp(invalue); break;
    case BIGINT : result = BigInt(invalue); break;
    case FLOAT : result = Field_element((double)(invalue)); break;
    case BIGFLOAT : result = BigFloat(invalue); break;
    case SLP : result = Slp(invalue); break;
    default : cout << "Error in Field_element(int,int), unknown type\n"; 
  }
  return result;
}

Field_element
make_Field_element(const integer& invalue, int fe_type) {
  Field_element result;
  switch (fe_type) {
    case BIGINT : result = BigInt(invalue); break;
    default : cout << "Error in make_Field_element(integer,int), unknown type\n"; 
  }
  return result;
}

Field_element
make_Field_element(double invalue, int fe_type) {
  Field_element result;
  switch (fe_type) {
    case SMALLINT : result = Field_element((int)invalue); break;
    case INTMODP : result = Field_element((unsigned)invalue); break;
    case BIGINT : result = BigInt((long)invalue); break;
    case FLOAT : result = Field_element(invalue); break;
    case BIGFLOAT : result = BigFloat(invalue); break;
    case SLP : result = Slp(invalue); break;
    default : cout << "Error in makeField_element, unknown type\n"; 
  }
  return result;
}

int
Field_element::i_value() const { 
  int result; 
  switch (type_of()) {
    case SMALLINT : result = ((SmallInt*)(this))->i_value(); break; 
    case INTMODP : result = ((IntModp*)(this))->i_value();  break;
    default : 
      cout << "Error in i_value() - called with non SmallInt or IntModp\n"; 
  }
  return result;
}
 
int
Field_element::i_value(int i) const { 
  int result; 
  switch (type_of()) {
    case MULTI_INTMODP : result = ((MultiIntModp*)(this))->i_value(i);  break;
    default : 
      cout << "Error in i_value() - called with non MultiIntModp\n"; 
  }
  return result;
}
