
/*************************************************************************

  APU R 2.0

  float.c

  (c) 1996, Regents of the University of California. All rights reserved. 

  Author : Ashutosh Rege (rege@cs.berkeley.edu)

*************************************************************************/
  
#include "float.h"

const char* Float::Class_name = "Float";

Float
operator + (const Float& a, const Float&b) {
  Float result; 
  result = Float(*((double*)a.frep) + *((double*)b.frep));
  return(result);
}

Float
operator - (const Float& a, const Float&b) {
  Float result; 
  result = Float(*((double*)a.frep) - *((double*)b.frep));
  return(result);
}

Float
operator - (const Float& a) {
  Float result; 
  result = Float( - (*((double*)a.frep)) );
  return(result);
}

Float
operator * (const Float& a, const Float&b) {
  Float result; 
  result = Float(*((double*)a.frep) * *((double*)b.frep));
  return(result);
}

Float
operator / (const Float& a, const Float&b) {
  Float result; 
  result = Float(*((double*)a.frep) / *((double*)b.frep));
  return(result);
}

Float
operator % (const Float& a, const Float&b) {
  Float result; 
  result = Float(fmod(*((double*)a.frep),*((double*)b.frep)));
  return(result);
}

Float
operator ^ (const Float& a, const Float&b) {
  Float result; 
  result = Float(pow(*((double*)a.frep),*((double*)b.frep)));
  return(result);
}

Float
operator ^ (const Float& a, int b) {
  Float result; 
  result = Float(pow(*((double*)a.frep),(double)(b)));
  return(result);
}

bool
operator == (const Float& a, const Float&b) {
  return(*((double*)a.frep) == *((double*)b.frep));
}

bool
operator != (const Float& a, const Float&b) {
  return(*((double*)a.frep) != *((double*)b.frep));
}

bool
operator < (const Float& a, const Float&b) {
  return(*((double*)a.frep) < *((double*)b.frep));
}

bool
operator <= (const Float& a, const Float&b) {
  return(*((double*)a.frep) <= *((double*)b.frep));
}

bool
operator > (const Float& a, const Float&b) {
  return(*((double*)a.frep) > *((double*)b.frep));
}

bool
operator >= (const Float& a, const Float&b) {
  return(*((double*)a.frep) >= *((double*)b.frep));
}

bool
is_zero(const Float& a) {
  if (*((double*)a.frep) == 0.0) return(true);
  else return(false);
}

int 
sign (const Float& a) {
  if (*((double*)a.frep) == 0.0) return(0);
  else if (*((double*)a.frep) < 0.0) return(-1);
  else return(1);
}
