
/*************************************************************************

  APU R 2.0

  intmodp.c

  (c) 1996, Regents of the University of California. All rights reserved. 

  Author : Ashutosh Rege (rege@cs.berkeley.edu)

*************************************************************************/
  
#include "intmodp.h"

const char* IntModp::Class_name = "IntModp";

IntModp 
operator+ (const IntModp& a, const IntModp& b) 
{
  IntModp result;
  /*
  int temp = (int)
   (fmod(((double)(a.i_value())+(double)(b.i_value())),(double)(a.prime_of())));
  */
  int temp = (a.i_value() + b.i_value()) % a.prime_of(); 
  result = IntModp(temp); 
  return(result);
}

IntModp 
operator- (const IntModp& a, const IntModp& b) 
{
  IntModp result;
  /*
  int temp = (int)
   (fmod(((double)(a.i_value())-(double)(b.i_value())),(double)(a.prime_of())));
  */
  int temp = (a.i_value() - b.i_value()) % a.prime_of(); 
  result = IntModp(temp); 
  return(result);
}

IntModp 
operator- (const IntModp& a)  // negation operator 
{
  IntModp result;
  /*
  int temp = (int)(fmod(- ((double)(a.i_value())), (double)(a.prime_of()))); 
  */
  int temp = (- a.i_value()) % a.prime_of(); 
  if (temp < 0) temp = temp + a.prime_of(); 
  result = IntModp(temp); 
  return(result);
}

IntModp 
operator* (const IntModp& a, const IntModp& b) 
{
  IntModp result;
  /*
  long temp; 
  temp = (int)(fmod(((double)(a.i_value())*(double)(b.i_value())),(double)(a.prime_of())));
  */
  unsigned temp = modmult((unsigned)(a.i_value()),(unsigned)(b.i_value()),
			  (unsigned)(a.prime_of()));   
  result = IntModp(temp);
  return(result);
}

IntModp 
operator/ (const IntModp& a, const IntModp& b) 
{
  IntModp result;
  int temp; 
  temp = finitef_inverse(b.i_value(),a.prime_of());
  // temp = (int) (finitef_mult(temp,a.i_value(),a.prime_of())); 
  unsigned temp2 = modmult((unsigned)(temp),(unsigned)(a.i_value()),
			   (unsigned)(a.prime_of()));
  result = IntModp(temp2);
  return(result);
}

IntModp 
operator% (const IntModp& a, const IntModp& b) 
{
  IntModp result;
  long temp;
  temp = a.i_value() % b.i_value(); 
  result = IntModp(temp);
  return(result);
}

IntModp 
operator^ (const IntModp& a, int b)
{
  IntModp result;
  result = IntModp(1);
  for (int i=0;i<b;i++) {
    result = result*a;
  }
  return(result);
}

IntModp 
operator^ (const IntModp& a, long int b)
{
  IntModp result;
  long i;
  result = IntModp(1);
  for (i=0;i<b;i++) {
    result = result*a;
  }
  return(result);
}

bool 
operator== (const IntModp& a, const IntModp& b) 
{
  bool result;
  if (a.i_value() == b.i_value()) result = true;
  else result = false;
  return(result);
}

bool 
operator!= (const IntModp& a, const IntModp& b) 
{
  bool result;
  if (a.i_value() != b.i_value()) result = true;
  else result = false;
  return(result);
}

/* Outputs a number a such that a*m = 1+p*b, that is, a is the inverse
of m mod p */
int
finitef_inverse(int m,int p)
{
    int aprime, bprime, a, b, c, d,q, r, t;

    aprime = b = 1;
    a = bprime = 0;
    c =  m; d =  p;
    q = c/d; r = c % d;
    while (r != 0) {
        c = d; d = r; t = aprime; aprime = a; a = t-q*a;
        t = bprime; bprime = b; b = t-q*b;
        q = c/d; r = c % d;
    }   /* end of while */

    return(a>=0? a : (a+p));
}

int 
finitef_mult(int a, int b, int p)
{
  int temp = (int) (fmod(((double)(a)*(double)(b)),(double)(p)));
  if (temp < 0) temp += p; 
  return temp; 
}

int 
finitef_add(int a, int b, int p)
{
  int temp = (a+b) % p; 
  if (temp < 0) temp += p; 
  return temp; 
}

int 
finitef_sub(int a, int b, int p)
{
  int temp = (a - b) % p; 
  if (temp < 0) temp += p; 
  return temp; 
}

bool 
is_zero (const IntModp& a) 
{
  if ((a.i_value() % a.prime_of()) == 0) 
    return(true); 
  else return(false);
}

