
/*************************************************************************

  APU R 2.0

  intmodp_mintmodp.c

  (c) 1996, Regents of the University of California. All rights reserved. 

  Author : Ashutosh Rege (rege@cs.berkeley.edu)

*************************************************************************/
  
#include "multi_intmodp.h"
#include "intmodp.h"
#include "intmodp_mintmodp.h"

/* Operators combining the IntModp field type with the MultiIntModp type. 
   The result of each operator will be an IntModp. If "a" is the IntModp
   operand and "b" the MultiIntModp operand, we look for the modp value
   in b which has the same prime as that in "a" and combine those two. 
*/

IntModp 
operator+ (const IntModp& a, const MultiIntModp& b) 
{
  IntModp result;
  int i,temp;
  for (i=0;i<MultiIntModp::CurrentNumPrimes;i++) { 
    if (a.prime_of() == b.prime_of(i)) { 
      temp = (a.i_value() + b.i_value(i)) % a.prime_of();
      break;
    }
  }
  if (i==MultiIntModp::CurrentNumPrimes) 
    printf("+(IntModp,MultiIntModp) - primes don't match\n");
  else result = IntModp(temp); 
  return(result);
}

IntModp 
operator+ (const MultiIntModp& a, const IntModp& b) 
{
  IntModp result;
  int i, temp;
  for (i=0;i<MultiIntModp::CurrentNumPrimes;i++) { 
    if (a.prime_of(i) == b.prime_of()) { 
      temp = (a.i_value(i) + b.i_value()) % b.prime_of();
      break;
    }
  }
  if (i==MultiIntModp::CurrentNumPrimes) 
    printf("+(MultiIntModp,IntModp) - primes don't match\n");
  else result = IntModp(temp); 
  return(result);
}

IntModp 
operator- (const IntModp& a, const MultiIntModp& b) 
{
  IntModp result;
  int i,temp;
  for (i=0;i<MultiIntModp::CurrentNumPrimes;i++) { 
    if (a.prime_of() == b.prime_of(i)) { 
      temp = (a.i_value() - b.i_value(i)) % a.prime_of();
      if (temp < 0) temp = temp+a.prime_of();
      break;
    }
  }
  if (i==MultiIntModp::CurrentNumPrimes) 
    printf("-(IntModp,MultiIntModp) - primes don't match\n");
  else result = IntModp(temp); 
  return(result);
}

IntModp 
operator- (const MultiIntModp& a, const IntModp& b) 
{
  IntModp result;
  int i,temp;
  for (i=0;i<MultiIntModp::CurrentNumPrimes;i++) { 
    if (a.prime_of(i) == b.prime_of()) { 
      temp = (a.i_value(i) - b.i_value()) % b.prime_of();
      if (temp < 0) temp += b.prime_of();
      break;
    }
  }
  if (i==MultiIntModp::CurrentNumPrimes) 
    printf("+(MultiIntModp,IntModp) - primes don't match\n");
  else result = IntModp(temp); 
  return(result);
}

IntModp 
operator* (const IntModp& a, const MultiIntModp& b) 
{
  IntModp result;
  int i,temp;
  for (i=0;i<MultiIntModp::CurrentNumPrimes;i++) { 
    if (a.prime_of() == b.prime_of(i)) { 
      temp = (int)modmult((unsigned)a.i_value(),(unsigned)b.i_value(i),
			 (unsigned)a.prime_of());
      break;
    }
  }
  if (i==MultiIntModp::CurrentNumPrimes) 
    printf("*(IntModp,MultiIntModp) - primes don't match\n");
  else result = IntModp(temp); 
  return(result);
}

IntModp 
operator* (const MultiIntModp& a, const IntModp& b) 
{
  IntModp result;
  int i,temp;
  for (i=0;i<MultiIntModp::CurrentNumPrimes;i++) { 
    if (a.prime_of(i) == b.prime_of()) { 
      temp = (int)modmult((unsigned)a.i_value(i),(unsigned)b.i_value(),
			 (unsigned)b.prime_of());
      break;
    }
  }
  if (i==MultiIntModp::CurrentNumPrimes) 
    printf("*(MultiIntModp,IntModp) - primes don't match\n");
  else result = IntModp(temp); 
  return(result);
}

IntModp 
operator/ (const IntModp& a, const MultiIntModp& b) 
{
  IntModp result;
  int i,temp;
  for (i=0;i<MultiIntModp::CurrentNumPrimes;i++) { 
    if (a.prime_of() == b.prime_of(i)) { 
      temp = finitef_inverse(b.i_value(i),a.prime_of());
      temp = (int)modmult((unsigned)a.i_value(),(unsigned)temp,
			 (unsigned)a.prime_of());
      break;
    }
  }
  if (i==MultiIntModp::CurrentNumPrimes) 
    printf("/(IntModp,MultiIntModp) - primes don't match\n");
  else result = IntModp(temp); 
  return(result);
}

IntModp 
operator/ (const MultiIntModp& a, const IntModp& b) 
{
  IntModp result;
  int i,temp;
  for (i=0;i<MultiIntModp::CurrentNumPrimes;i++) { 
    if (a.prime_of(i) == b.prime_of()) { 
      temp = finitef_inverse(b.i_value(),b.prime_of());
      temp = (int)modmult((unsigned)a.i_value(i),(unsigned)temp,
			 (unsigned)b.prime_of());
      break;
    }
  }
  if (i==MultiIntModp::CurrentNumPrimes) 
    printf("/(MultiIntModp,IntModp) - primes don't match\n");
  else result = IntModp(temp); 
  return(result);
}
