
/*************************************************************************

  APU R 2.0

  mintmodp.c

  (c) 1996, Regents of the University of California. All rights reserved. 

  Author : Ashutosh Rege (rege@cs.berkeley.edu)

*************************************************************************/
  
#include "multi_intmodp.h"
#include "bigint.h"
#include "crt.h"

const char* MultiIntModp::Class_name = "MultiIntModp";

MultiIntModp 
operator+ (const MultiIntModp& a, const MultiIntModp& b) 
{
  MultiIntModp result;
  int temp[MULTI_NUMPRIMES]; 
  for (int i=0;i<MultiIntModp::CurrentNumPrimes;i++) { 
  /*
    temp[i] = (int)
   (fmod(((double)(a.i_value(i))+(double)(b.i_value(i))),(double)(a.prime_of(i))));
   */
   temp[i] = (a.i_value(i) + b.i_value(i)) % a.prime_of(i);
  }
  result = MultiIntModp(temp); 
  return(result);
}

MultiIntModp 
operator- (const MultiIntModp& a, const MultiIntModp& b) 
{
  MultiIntModp result;
  int temp[MULTI_NUMPRIMES]; 
  for (int i=0;i<MultiIntModp::CurrentNumPrimes;i++) { 
    /*
    temp[i] = (int)
   (fmod(((double)(a.i_value(i))-(double)(b.i_value(i))),(double)(a.prime_of(i))));
   */
   temp[i] = (a.i_value(i) - b.i_value(i)) % a.prime_of(i);
    if (temp[i] < 0) temp[i] = temp[i] + a.prime_of(i); 
  }
  result = MultiIntModp(temp); 
  return(result);
}

MultiIntModp 
operator- (const MultiIntModp& a)  // negation operator 
{
  MultiIntModp result;
  int temp[MULTI_NUMPRIMES]; 
  for (int i=0;i<MultiIntModp::CurrentNumPrimes;i++) { 
    /*
    temp[i] = (int) (fmod(((double)(-a.i_value(i))),(double)(a.prime_of(i))));
    */
    temp[i] = (- a.i_value(i)) % a.prime_of(i); 
    if (temp[i] < 0) temp[i] = temp[i] + a.prime_of(i); 
  }
  result = MultiIntModp(temp); 
  return(result);
}

MultiIntModp 
operator* (const MultiIntModp& a, const MultiIntModp& b) 
{
  int temp[MULTI_NUMPRIMES]; 
  for (int i=0;i<MultiIntModp::CurrentNumPrimes;i++) { 
  /* 
    temp[i] = (int)
   (fmod(((double)(a.i_value(i))*(double)(b.i_value(i))),(double)(a.prime_of(i))));
    */
    temp[i] = (int) modmult((unsigned)(a.i_value(i)),(unsigned)(b.i_value(i)),
			  (unsigned)(a.prime_of(i)));   
  }
  MultiIntModp result = MultiIntModp(temp); 
  return(result);
}

MultiIntModp 
operator/ (const MultiIntModp& a, const MultiIntModp& b) 
{
  MultiIntModp result;
  int temp[MULTI_NUMPRIMES]; 
  for (int i=0;i<MultiIntModp::CurrentNumPrimes;i++) { 
    temp[i] = (int)finitef_inverse(b.i_value(i),a.prime_of(i)); 
    // temp[i] = (int)(finitef_mult(temp[i],a.i_value(i),a.prime_of(i))); 
    temp[i] = (int)modmult((unsigned)(temp[i]),(unsigned)(a.i_value(i)),
			   (unsigned)(a.prime_of(i)));
  }
  result = MultiIntModp(temp); 
  return(result);
}

MultiIntModp 
operator% (const MultiIntModp& a, const MultiIntModp& b) 
{
  MultiIntModp result;
  int temp[MULTI_NUMPRIMES]; 
  for (int i=0;i<MultiIntModp::CurrentNumPrimes;i++) { 
    temp[i] = a.i_value(i) % b.i_value(i); 
  }
  result = MultiIntModp(temp);
  return(result);
}

MultiIntModp 
operator^ (const MultiIntModp& a, int b)
{
  MultiIntModp result;
  result = MultiIntModp(1);
  for (int i=0;i<b;i++) {
    result = result*a;
  }
  return(result);
}

MultiIntModp 
operator^ (const MultiIntModp& a, long int b)
{
  MultiIntModp result;
  result = MultiIntModp(1);
  for (int i=0;i<b;i++) {
    result = result*a;
  }
  return(result);
}

bool 
operator== (const MultiIntModp& a, const MultiIntModp& b) 
{
  for (int i=0;i<MultiIntModp::CurrentNumPrimes;i++) { 
    if (a.i_value(i) != b.i_value(i)) 
      return false; 
  }
  return(true);
}

bool 
operator!= (const MultiIntModp& a, const MultiIntModp& b) 
{
  return(!(a==b));
}

bool 
is_zero (const MultiIntModp& a) 
{
  for (int i=0;i<MultiIntModp::CurrentNumPrimes;i++) { 
    if (a.i_value(i) != 0) 
      return false; 
  }
  return(true);
}

int 
sign (const MultiIntModp& a) 
{
  if (is_zero(a)) return(0); 
  else { 
    BigInt value = multi_crt(a); 
    return(sign(value)); 
  }
}
