
/*************************************************************************

  APU R 2.0

  mintmodp_smallint.c

  (c) 1996, Regents of the University of California. All rights reserved. 

  Author : Ashutosh Rege (rege@cs.berkeley.edu)

*************************************************************************/
  
#include "smallint.h"
#include "multi_intmodp.h"
#include "mintmodp_smallint.h"

MultiIntModp 
operator+ (const MultiIntModp& a, const SmallInt& b) 
{
  MultiIntModp result;
  int temp[MULTI_NUMPRIMES]; 
  for (int i=0;i<MultiIntModp::CurrentNumPrimes;i++) { 
  /*
    temp[i] = (int)
  (fmod(((double)(a.i_value(i))+(double)(b.i_value())),(double)(a.prime_of(i))));
  */
    temp[i] = (a.i_value(i) + b.i_value() % a.prime_of(i)); 
  }
  result = MultiIntModp(temp); 
  return(result);
}

MultiIntModp 
operator+ (const SmallInt& a, const MultiIntModp& b) 
{
  MultiIntModp result;
  int temp[MULTI_NUMPRIMES]; 
  for (int i=0;i<MultiIntModp::CurrentNumPrimes;i++) { 
  /*
   temp[i] = (int)
  (fmod(((double)(a.i_value())+(double)(b.i_value(i))),(double)(b.prime_of(i))));
  */
    temp[i] = (a.i_value() + b.i_value(i) % b.prime_of(i)); 
  }
  result = MultiIntModp(temp); 
  return(result);
}

MultiIntModp 
operator- (const MultiIntModp& a, const SmallInt& b) 
{
  MultiIntModp result;
  int temp[MULTI_NUMPRIMES]; 
  for (int i=0;i<MultiIntModp::CurrentNumPrimes;i++) { 
    /*
    temp[i] = (int)
  (fmod(((double)(a.i_value(i))-(double)(b.i_value())),(double)(a.prime_of(i))));
  */
    temp[i] = (a.i_value(i) - b.i_value()) % a.prime_of(i); 
    if (temp[i] < 0) temp[i] = temp[i] + a.prime_of(i); 
  }
  result = MultiIntModp(temp); 
  return(result);
}

MultiIntModp 
operator- (const SmallInt& a, const MultiIntModp& b) 
{
  MultiIntModp result;
  int temp[MULTI_NUMPRIMES]; 
  for (int i=0;i<MultiIntModp::CurrentNumPrimes;i++) { 
   /*
   temp[i] = (int)
  (fmod(((double)(a.i_value())-(double)(b.i_value(i))),(double)(b.prime_of(i))));
  */
    temp[i] = (a.i_value() - b.i_value(i)) % b.prime_of(i); 
    if (temp[i] < 0) temp[i] = temp[i] + b.prime_of(i); 
  }
  result = MultiIntModp(temp); 
  return(result);
}

MultiIntModp 
operator* (const MultiIntModp& a, const SmallInt& b) 
{
  MultiIntModp result;
  MultiIntModp temp = MultiIntModp(b.i_value());  
  result = a*temp; 
  return(result);
}

MultiIntModp 
operator* (const SmallInt& a, const MultiIntModp& b) 
{
  MultiIntModp result;
  MultiIntModp temp = MultiIntModp(a.i_value());  
  result = temp*b; 
  return(result);
}

MultiIntModp 
operator/ (const MultiIntModp& a, const SmallInt& b) 
{
  MultiIntModp result;
  int temp[MULTI_NUMPRIMES]; 
  for (int i=0;i<MultiIntModp::CurrentNumPrimes;i++) { 
    temp[i] = finitef_inverse(b.i_value(),a.prime_of(i)); 
    temp[i] = (int) modmult((unsigned)temp[i],(unsigned)a.i_value(i),
			    (unsigned)a.prime_of(i)); 
  }
  result = MultiIntModp(temp); 
  return(result);
}

MultiIntModp 
operator/ (const SmallInt& a, const MultiIntModp& b) 
{
  MultiIntModp result;
  int temp[MULTI_NUMPRIMES]; 
  for (int i=0;i<MultiIntModp::CurrentNumPrimes;i++) { 
    temp[i] = finitef_inverse(b.i_value(i),b.prime_of(i)); 
    temp[i] = (int) modmult((unsigned)temp[i],(unsigned)b.i_value(i),
			    (unsigned)b.prime_of(i)); 
  }
  result = MultiIntModp(temp); 
  return(result);
}

