
/*************************************************************************

  APU R 2.0

  ratfloatf.c

  (c) 1996, Regents of the University of California. All rights reserved. 

  Author : Ashutosh Rege (rege@cs.berkeley.edu)

*************************************************************************/
  

#include "ratfloatf.h"

const char* RatFloatf::Class_name = "RatFloatf";

RatFloatf
operator + (const RatFloatf& a, const RatFloatf& b) {
  float_filter numer = numer_of(a)*denom_of(b) + numer_of(b)*denom_of(a); 
  float_filter denom = denom_of(a)*denom_of(b); 
  RatFloatf result = RatFloatf(numer,denom); 
  return(result);
}

RatFloatf
operator - (const RatFloatf& a, const RatFloatf& b) {
  float_filter numer = numer_of(a)*denom_of(b) - numer_of(b)*denom_of(a); 
  float_filter denom = denom_of(a)*denom_of(b); 
  RatFloatf result = RatFloatf(numer,denom); 
  return(result);
}

RatFloatf
operator - (const RatFloatf& a) {
  float_filter numer = float_filter(integer(-1))*numer_of(a);
  RatFloatf result = RatFloatf(numer,denom_of(a)); 
}

RatFloatf
operator * (const RatFloatf& a, const RatFloatf& b) {
  float_filter numer = numer_of(a)*numer_of(b);
  float_filter denom = denom_of(a)*denom_of(b); 
  RatFloatf result = RatFloatf(numer,denom); 
  return(result);
}

RatFloatf
operator / (const RatFloatf& a, const RatFloatf& b) {
  float_filter numer = numer_of(a)*denom_of(b);
  float_filter denom = denom_of(a)*numer_of(b); 
  RatFloatf result = RatFloatf(numer,denom); 
  return(result);
}

RatFloatf
operator ^ (const RatFloatf& a, int b) {
  if (b == 0) return(RatFloatf(1));
  else if (b == 1) return(a);
  else {
    RatFloatf result = a; 
    for (int i=0;i<b-1;i++) 
      result = result*a; 
    return(result);
  }
}

bool
operator < (const RatFloatf& a, const RatFloatf& b) {
  RatFloatf temp = a - b; 
  int diff_sign = sign(numer_of(temp))*sign(denom_of(temp)); 
  if (diff_sign < 0) return(true);
  else return(false); 
}

bool
operator <= (const RatFloatf& a, const RatFloatf& b) {
  RatFloatf temp = a - b; 
  int diff_sign = sign(numer_of(temp))*sign(denom_of(temp)); 
  if (diff_sign <= 0) return(true);
  else return(false); 
}

bool
operator > (const RatFloatf& a, const RatFloatf& b) {
  RatFloatf temp = a - b; 
  int diff_sign = sign(numer_of(temp))*sign(denom_of(temp)); 
  if (diff_sign > 0) return(true);
  else return(false); 
}

bool
operator >= (const RatFloatf& a, const RatFloatf& b) {
  RatFloatf temp = a - b; 
  int diff_sign = sign(numer_of(temp))*sign(denom_of(temp)); 
  if (diff_sign >= 0) return(true);
  else return(false); 
}

int 
sign (const RatFloatf& a) {
  int diff_sign = sign(numer_of(a))*sign(denom_of(a)); 
  if (abs(diff_sign) > 1) return(DONT_KNOW);  
  else return(diff_sign); 
}
