
/*************************************************************************

  APU R 2.0

  adfloat.h

  (c) 1996, Regents of the University of California. All rights reserved. 

  Author : Ashutosh Rege (rege@cs.berkeley.edu)

*************************************************************************/
  

#if !defined(ADFLOAT_HDR)
#define ADFLOAT_HDR

#include "globals.h"
#include "fielddef.h"

class AdFloat : public Field_element {
  private :
    static const char* Class_name;
  public : 

    AdFloat() { type = ADFLOAT; frep = 0; }; 
    AdFloat(float);
    AdFloat(double);
    AdFloat(int,double*);

    int exp_size() const; 
    double* expan_of() const; 
    double& operator[](int); 

    friend AdFloat operator + (const AdFloat&, const AdFloat&);
    friend AdFloat operator - (const AdFloat&, const AdFloat&);
    friend AdFloat operator * (const AdFloat&, const AdFloat&);
    friend AdFloat operator / (const AdFloat&, const AdFloat&);

    friend AdFloat operator - (const AdFloat&);
    friend AdFloat operator ^ (const AdFloat&, int);
    friend ostream& operator << (ostream&, AdFloat); 
};

typedef struct adfloat_node {
  int exp_size;		/* number of terms in expansion */ 
  double *expan;        /* the expansion */
} AdFloat_node; 

inline AdFloat::AdFloat(float a) {
  type = ADFLOAT;
  frep = (void*) (new AdFloat_node);
  ((AdFloat_node*)frep)->exp_size = 1;
  ((AdFloat_node*)frep)->expan = new double[1];
  ((AdFloat_node*)frep)->expan[0] = (double)(a);
}

inline AdFloat::AdFloat(double a) {
  type = ADFLOAT;
  frep = (void*) (new AdFloat_node);
  ((AdFloat_node*)frep)->exp_size = 1;
  ((AdFloat_node*)frep)->expan = new double[1];
  ((AdFloat_node*)frep)->expan[0] = a;
}

inline AdFloat::AdFloat(int size, double* expan) {
  type = ADFLOAT;
  frep = (void*) (new AdFloat_node);
  ((AdFloat_node*)frep)->exp_size = size;
  ((AdFloat_node*)frep)->expan = expan; 
}

inline int AdFloat::exp_size() const { 
  return(((AdFloat_node*)frep)->exp_size); 
}

inline int exp_size(const AdFloat& a) { 
  return(a.exp_size()); 
}

inline double* AdFloat::expan_of() const { 
  return(((AdFloat_node*)frep)->expan); 
}

inline double* expan_of(const AdFloat& a) { 
  return(a.expan_of()); 
}

inline double& AdFloat::operator[](int ix) {
  return(((AdFloat_node*)frep)->expan[ix]); 
}

#include "adaptive_float.h"
inline ostream& operator << (ostream& os, AdFloat a) {
  int sz = exp_size(a); 
  for (int i=0;i<sz;i++) {
    doubleprint(a[i]); 
    cout << "\n"; 
  }
}

#endif
