
/*************************************************************************

  APU R 2.0

  field_unaryop.h

  (c) 1996, Regents of the University of California. All rights reserved. 

  Author : Ashutosh Rege (rege@cs.berkeley.edu)

*************************************************************************/
  

#if !defined(FIELDAUX3_HDR)
#define FIELDAUX3_HDR

#include "fielddef.h"
#include "unaryop.h"

template<class UnaryOp>
void
unary_op (Field_element& result, UnaryOp op, const Field_element& fe) 
{
  switch(fe.type_of()){
    case SMALLINT : 
      UnaryOp(result,(SmallInt&)(fe)); break;
    case INTMODP : 
      UnaryOp(result,(IntModp&)(fe)); break;
    case BIGINT : 
      UnaryOp(result,(BigInt&)(fe)); break;
    case FLOAT : 
      UnaryOp(result,(Float&)(fe)); break;
    case BIGFLOAT : 
      UnaryOp(result,(BigFloat&)(fe)); break;
    case RATFLOATF : 
      UnaryOp(result,(RatFloatf&)(fe)); break;
    case SLP : 
      UnaryOp(result,(Slp&)(fe)); break;
    default : break;
  }
}

inline Field_element 
operator - (const Field_element& a) {
  Negate op; Field_element result;
  unary_op(result,op,a);
  return(result);
}

template<class BoolUnaryOp>
void
bool_unary_op (bool& result, BoolUnaryOp op, const Field_element& fe) 
{
  switch(fe.type_of()){
    case SMALLINT : 
      BoolUnaryOp(result,(SmallInt&)(fe)); break;
    case INTMODP : 
      BoolUnaryOp(result,(IntModp&)(fe)); break;
    case BIGINT : 
      BoolUnaryOp(result,(BigInt&)(fe)); break;
    case FLOAT : 
      BoolUnaryOp(result,(Float&)(fe)); break;
    case BIGFLOAT : 
      BoolUnaryOp(result,(BigFloat&)(fe)); break;
    case RATFLOATF : 
      BoolUnaryOp(result,(RatFloatf&)(fe)); break;
    case SLP : 
      BoolUnaryOp(result,(Slp&)(fe)); break;
    default : break;
  }
}

inline bool 
is_zero(const Field_element& a) {
  Is_Zero op; bool result;
  bool_unary_op(result,op,a);
  return(result);
}

template<class IntUnaryOp>
void
int_unary_op (int& result, IntUnaryOp op, const Field_element& fe) 
{
  switch(fe.type_of()){
    case SMALLINT : 
      IntUnaryOp(result,(SmallInt&)(fe)); break;
    case INTMODP : 
      IntUnaryOp(result,(IntModp&)(fe)); break;
    case BIGINT : 
      IntUnaryOp(result,(BigInt&)(fe)); break;
    case FLOAT : 
      IntUnaryOp(result,(Float&)(fe)); break;
    case BIGFLOAT : 
      IntUnaryOp(result,(BigFloat&)(fe)); break;
    case RATFLOATF : 
      IntUnaryOp(result,(RatFloatf&)(fe)); break;
    case SLP : 
      IntUnaryOp(result,(Slp&)(fe)); break;
    default : break;
  }
}

inline int
sign(const Field_element& a) {
  Sign op; int result;
  int_unary_op(result,op,a); 
  return(result);
}

#endif
