
/*************************************************************************

  APU R 2.0

  list.h

  (c) 1996, Regents of the University of California. All rights reserved. 

  Author : Ashutosh Rege (rege@cs.berkeley.edu)

*************************************************************************/
  

#if !defined(LISTHDR) 
#define LISTHDR

#include "globals.h"

template <class Type>
class ListNode {
  private : 
    Type item;
    ListNode<Type> *next;
  public : 
    ListNode(const Type&);

    ListNode<Type>* next_of();
    void set_next_of(ListNode<Type>*); 
    Type item_of(); 

    friend class List<Type>;
    friend ostream& operator << (ostream&, ListNode<Type>&);
};

template <class Type>
inline
ListNode<Type>::ListNode(const Type& t) : item(t) {next = 0;}

template <class Type>
inline ListNode<Type>* ListNode<Type>::next_of() { return(next); }

template <class Type> 
inline void ListNode<Type>::set_next_of(ListNode<Type>* item_p) {
  next = item_p; 
}

template <class Type>
inline Type ListNode<Type>::item_of() { return(item); }

template <class Type>
inline Type item_of(ListNode<Type> lnode) { return(ln.item_of()); }

template <class Type>
class List {
  private : 
    ListNode<Type> *front; 
    ListNode<Type> *back;
    int list_size; 
  public : 
    List() {front = back = 0; list_size = 0;}

    void add(const Type&); 
    void append(const Type&); 
    void prepend(const Type&); 
    bool is_empty() const { if (front == 0) return(true); else return(false); }
    ListNode<Type>* front_of() const; 
    void set_front_of(ListNode<Type>*); 
    ListNode<Type>* delete_head(); 
    ListNode<Type>* back_of() const; 
    friend ostream& operator << (ostream&, const List<Type>&); 
    int size_of() const { if (is_empty()) return(0); else return(list_size); }
    void inc_size(int n=1) { list_size += n; } 
    void dec_size(int n=1) { list_size -= n; } 

    Type* find(char*); // Find string and return pointer to node. 
		       // Implemented for List<Variable> type in variable.h. 
};

template <class Type>
inline
void List<Type>::add(const Type &val)
{
  ListNode<Type> *pt = new ListNode<Type>(val); 

  if (is_empty()) { 
    front = back = pt; 
    list_size = 1; 
  } 
  else {
    back->next = pt; 
    back = pt; 
    pt->next = 0; 
    list_size++; 
  }
}

template <class Type>
inline
void List<Type>::append(const Type& val) 
{
  add(val);
}

template <class Type>
inline
void List<Type>::prepend(const Type &val)
{
  ListNode<Type> *pt = new ListNode<Type>(val); 

  if (is_empty()) { 
    front = back = pt; 
    list_size = 1; 
  } 
  else {
    pt->next = front; 
    front = pt; 
    list_size++; 
  }
}

template <class Type>
inline ListNode<Type>* List<Type>::front_of() const {
  return(front);
}

template <class Type>
inline
void List<Type>::set_front_of(ListNode<Type>* item_p) { 
  if (item_p == 0) { 
    front = item_p; back = item_p; 
    list_size = 0;   
  }
  else { 
    item_p->set_next_of(front->next_of()); 
    if (back == front) {
      front = item_p; 
      back = front; 
    } 
    else front = item_p; 
    list_size++; 
  }
}

template<class Type>
inline
ListNode<Type>* List<Type>::delete_head() { 
  ListNode<Type>* result = front; 
  if (front != 0) {
    front = front->next_of(); 
    list_size--; 
  }
  return result; 
}

template <class Type>
inline ListNode<Type>* head_of(const List<Type>& list) {
  return(list.front_of());
}

template <class Type>
inline ListNode<Type>* List<Type>::back_of() const {
  return(back);
}

template <class Type>
inline ListNode<Type>* tail_of(const List<Type>& list) {
  return(list.back_of());
}

template <class Type>
inline ostream& 
operator << (ostream& os, ListNode<Type> &ln) {
  os << ln.item;
  return os;
}

template <class Type>
inline ostream&
operator << (ostream& os, const List<Type> &list) {
  ListNode<Type> *p;
  for (p = list.front; p; p = p->next_of()) 
    os << *p << "\n";
  return os;
}

template<class Type> 
inline List<Type> concat(List<Type> l1, List<Type> l2) { 
  List<Type> result; 
  ListNode<Type>* ptr;
  for (ptr=l1.front_of();ptr!=0;ptr=ptr->next_of()) { 
    result.append(ptr->item_of()); 
  } 
  for (ptr=l2.front_of();ptr!=0;ptr=ptr->next_of()) { 
    result.append(ptr->item_of()); 
  } 
  return result; 
} 

template<class Type> 
inline List<Type> copy_list(List<Type> list) { 
  List<Type> result; 
  ListNode<Type>* ptr; 

  for (ptr=list.front_of();ptr!=0;ptr=ptr->next_of()) { 
    result.append((ptr->item_of()));
  } 
  return result;
} 

#endif
