
/*************************************************************************

  APU R 2.0

  variable.h

  (c) 1996, Regents of the University of California. All rights reserved. 

  Author : Ashutosh Rege (rege@cs.berkeley.edu)

*************************************************************************/
  

#if !defined(VARIABLEHDR)
#define VARIABLEHDR

#include "fielddef.h"
#include "array.h"
#include "list.h"

class Slp; 

class Variable : public Field_element {
  private :
    static const char* Class_name;
  public : 
    Variable(); 
    Variable(char*);
    int index() const;         // get the index;
    void index(int);           // set the index; 
    char* name() const;        // get the name; 
    void name(char *);         // set the name; 
    Slp slp_of() const;        // get the slp corresponding to the variable
    Slp* slpPtr_of() const;    // get slp pointer; 
    void slp(Slp); 	       // set slp; 
    Field_element value();     // get value;
    void value(Field_element); // set value; 
    struct variable_node* frep_of();  // get the representation

    friend ostream& operator << (ostream&, const Variable&);

};

typedef struct variable_node {
  int index; 
  char* name;
  Field_element* value; 
  class Slp* slp;
} Variable_node; 

inline Variable::Variable() {
  type = VAR;
  frep = (void*) (new Variable_node);
  ((Variable_node*)frep)->index = -1;
  ((Variable_node*)frep)->name = "";
  ((Variable_node*)frep)->value = 0;
  ((Variable_node*)frep)->slp = 0;
}

inline Variable::Variable(char* str) {
  type = VAR; 
  frep = (void*) (new Variable_node);
  ((Variable_node*)frep)->index = -1;
  ((Variable_node*)frep)->name = new char[strlen(str)+1]; 
  strcpy(((Variable_node*)frep)->name,str);
  ((Variable_node*)frep)->value = 0;
  ((Variable_node*)frep)->slp = 0;
}

inline struct variable_node* Variable::frep_of() {
  return((Variable_node*)frep);
}

inline int Variable::index() const {
  if (frep == 0) 
    cout << "Variable.index() : Illegal Variable call - no frep\n";
  else return(((Variable_node*)frep)->index);
}

inline int index(const Variable& var) {
  return(var.index());
}

inline void Variable::index(int ix) {
  if (frep == 0) 
    cout << "Variable.index() : Illegal Variable call - no frep\n";
  else ((Variable_node*)frep)->index = ix;
}

inline char* Variable::name() const {
  if (frep == 0) 
    cout << "Variable.name() : Illegal Variable call - no frep\n";
  else return(((Variable_node*)frep)->name);
}

inline char* name(const Variable& var) {
  return(var.name());
}

inline void Variable::name(char* str) {
  ((Variable_node*)frep)->name = new char[strlen(str)+1];
  strcpy(((Variable_node*)frep)->name,str);
}

inline Slp* Variable::slpPtr_of() const {
  if (frep == 0) {
    cout << "Variable.name() : Illegal Variable call - no frep\n";
    return(0);
  }
  else return(((Variable_node*)frep)->slp);
}

inline Slp* slpPtr_of(const Variable& var) {
  return(var.slpPtr_of());
}

inline Field_element Variable::value() {
  if (frep == 0) {
    cout << "Variable.name() : Illegal Variable call - no frep\n";
    return(0);
  }
  else return(*(((Variable_node*)frep)->value));
}

inline void Variable::value(Field_element a) {
  if (frep == 0) {
    cout << "Variable.name() : Illegal Variable call - no frep\n";
  }
  else { 
    if ((((Variable_node*)frep)->value) == 0)   
      (((Variable_node*)frep)->value) = new Field_element; 
    *(((Variable_node*)frep)->value) = a; 
  }
}

inline int
Array<Variable>::find(char* str) {
  for (int i=0;i<size;i++) {
    if (!strcmp((array[i]).name(),str)) {
      return(i);
    }
  }
  return(-1);
}

inline Variable*
List<Variable>::find(char* str) {
  ListNode<Variable> *temp; int i;
  temp = front;
  i = 0; 
  while (temp != 0) {
    if (!strcmp(temp->item.name(),str)) 
      return(&(temp->item)); 
    else i++;
    temp = temp->next;
  }
  return(0);
}

#endif
