
/*************************************************************************

  APU R 2.0

  array.h

  (c) 1996, Regents of the University of California. All rights reserved. 

  Author : Ashutosh Rege (rege@cs.berkeley.edu)

*************************************************************************/
  

#if !defined(ARRAY_HDR) 
#define ARRAY_HDR

#include "arith_util.h"

template <class Type>
class Array {
  private : 
    Type* array;
    int size;
  public : 
    Array() : array(0), size(0) {}
    Array(int sz) : array(new Type[sz]), size(sz) {}

    Type& operator[](int ix) { return array[ix]; } 
    Type& operator[](int ix) const { return array[ix]; } 
    int size_of() const { return size; } 
    void set_size(int sz) { size = sz; } 
    bool is_empty() const 
       {if ((array == 0) || (size == 0)) return(true); else return(false);}

    friend Type max_of(const Array<Type>&);
    friend ostream& operator << (ostream&, const Array<Type>&); 
    friend void copy_array(Array<Type>&, const Array<Type>&); 
    friend void partial_copy_array(Array<Type>&, const Array<Type>&,int); 

    int find(char*); // find char* and return index in array, else -1
		     // Defined for Array<Variable> type.
};

template <class Type>
inline int size_of(const Array<Type>& array) {
  return(array.size_of());
}

template <class Type> 
inline bool is_empty(const Array<Type>& array) {
  return (array.is_empty());
}

/*
template <class Type> 
inline Type Array<Type>::max_of() const {
  if (size == 0) {
    cout << "max_of(Array) called with null array\n"; 
  }
  else {
    Type result = array[0];
    for (int i=1;i<size;i++) {
      if (array[i] > result) 
	result = array[i];
    }
    return(result);
  }
}

template <class Type> 
inline Type max_of(const Array<Type>& arr) {
  return(arr.max_of());
}
*/

template <class Type> 
inline Type max_of(const Array<Type>& arr) {
  if (arr.size == 0) {
    cout << "max_of(Array) called with null array\n"; 
  }
  else {
    Type result = arr.array[0];
    for (int i=1;i<arr.size;i++) {
      if (arr.array[i] > result) 
	result = arr.array[i];
    }
    return(result);
  }
}

template <class Type> 
inline Array<Type> add_to_array(const Array<Type>& arr, Type num) {
  int dim = size_of(arr); 
  Array<Type> result = Array<Type>(dim); 
  for (int i=0;i<dim;i++) 
    result[i] = arr[i] + num;

  return(result);
}

template <class Type> 
inline Array<Type> sort(const Array<Type>& arr) {
  int dim = size_of(arr); 
  Array<Type> result = Array<Type>(dim); 
  for (int i=0;i<dim;i++)
    result[i] = arr[i];
  for (int i=0;i<dim;i++) {
    for (int j=i+1;j<dim;j++) {
      if (result[j] < result[i]) {
	Type temp = result[i]; 
	result[i] = result[j];
	result[j] = temp;
      }
    }
  }
  return(result);
}

template <class Type>
inline Array<Type> concat(const Array<Type>& a1, const Array<Type>& a2) {
  int dim1 = size_of(a1); int dim2 = size_of(a2);
  Array<Type> result = Array<Type>(dim1+dim2);
  for (int i=0;i<dim1;i++) 
    result[i] = a1[i];
  for (int i=dim1;i<dim1+dim2;i++) 
    result[i] = a2[i-dim1];
  return(result);
}

template <class Type>
ostream& operator << (ostream& os, const Array<Type>& M)
{
  int dim = size_of(M); 
  for (int i=0;i<dim;i++) {
      os << M[i] << "  ";
  }
  os << "\n";
  return os; 
}

template <class Type> 
void copy_array (Array<Type>& to, const Array<Type>& from) { 
  int dim1 = size_of(to); 
  int dim2 = size_of(from); 
  if (dim2 < dim1) { 
    cout << "Error in copy_array - mismatched dimensions\n"; 
    return; 
  } 
  for (int i=0;i<dim1;i++) 
    to[i] = from[i]; 
}

template <class Type> 
void partial_copy_array (Array<Type>& to, const Array<Type>& from, int dim) { 
  int dim1 = size_of(to); 
  int dim2 = size_of(from); 
  if (dim1 < dim) { 
    cout << "Error in partial_copy_array - mismatched dimensions\n"; 
    return; 
  } 
  for (int i=0;i<dim;i++) 
    to[i] = from[i]; 
}

Array<float> convert_to_float(Array<int>);
Array<int> convert_to_int(Array<float>);

#endif
