/*************************************************************************

  APU R 2.0

  bigint.h

  (c) 1996, Regents of the University of California. All rights reserved. 

  Author : Ashutosh Rege (rege@cs.berkeley.edu)

*************************************************************************/
  

#if !defined(BIGINTHDR)
#define BIGINTHDR

#include "globals.h"

#include "fielddef.h"
#include "integer.h"

class BigInt : public Field_element {
  private : 
    static const char* Class_name; 
  public : 

    BigInt(); 
    // ~BigInt(); 
    BigInt(int);
    BigInt(unsigned int);
    BigInt(long); 
    BigInt(const integer&); // LEDA "integer" type
    BigInt& operator=(const BigInt& x) 
    { type = x.type; 
      if (frep != 0) 
        *((integer*)(frep)) = *((integer*)(x.frep)); 
      else frep = x.frep; 
    }
    BigInt& operator=(int x) { type = BIGINT;
      if (frep != 0) { 
	*((integer*)(frep)) = integer(x);
      }
      else frep = new integer(x); 
    }
    BigInt& operator=(unsigned int x) { type = BIGINT;
      if (frep != 0) *((integer*)(frep)) = integer(x);
      else {frep = new integer(x); } 
    }
    const char* class_name(); 
    integer i_value() const;
    void print();

    friend BigInt operator + (const BigInt&, const BigInt&);
    friend BigInt operator - (const BigInt&, const BigInt&);
    friend BigInt operator * (const BigInt&, const BigInt&);
    friend BigInt operator / (const BigInt&, const BigInt&);

    friend BigInt operator % (const BigInt&, const BigInt&);
    friend BigInt operator ^ (const BigInt&, int);

    friend BigInt operator - (const BigInt&); // negation

    friend bool operator == (const BigInt&, const BigInt&);
    friend bool operator != (const BigInt&, const BigInt&);
    friend bool operator  < (const BigInt&, const BigInt&);
    friend bool operator <= (const BigInt&, const BigInt&);
    friend bool operator  > (const BigInt&, const BigInt&);
    friend bool operator >= (const BigInt&, const BigInt&);

    friend bool is_zero(const BigInt&);
    friend int sign(const BigInt&);
}; 

/*
inline BigInt::~BigInt() {
  if (frep != 0) 
    ~(*((integer*)frep));
}
*/

inline BigInt::BigInt() {
  type = BIGINT;
  frep = (void*) (new integer(0)); 
  // *((integer*)frep) = integer(0); 
}

inline BigInt::BigInt(int a) {
  type = BIGINT;
  frep = (void*) (new integer(a)); 
  // *((integer*)frep) = integer(a); 
}

inline BigInt::BigInt(unsigned int a) {
  type = BIGINT;
  frep = (void*) (new integer); 
  *((integer*)frep) = integer(a); 
}

inline BigInt::BigInt(long a) {
  type = BIGINT;
  frep = (void*) (new integer); 
  *((integer*)frep) = integer(a); 
}

inline BigInt::BigInt(const integer& a) {
  type = BIGINT; 
  frep = (void*)(new integer); 
  *((integer*)frep) = a; 
}

inline const char* BigInt::class_name() {
  return(Class_name); 
}

inline integer BigInt::i_value() const {
  return(*((integer*)frep));
}

inline void BigInt::print() {
  Print(*((integer*)frep),cout); 
}

#endif
