/*************************************************************************

  APU R 2.0

  bigint.h

  (c) 1996, Regents of the University of California. All rights reserved. 

  Author : Ashutosh Rege (rege@cs.berkeley.edu)

*************************************************************************/
  

#if !defined(BIGINTHDR)
#define BIGINTHDR

#include "globals.h"

#include "fielddef.h"
#include "integer.h"

extern int BigIntMAlloc;
class BigIntM : public Field_element {
  private : 
    static const char* Class_name; 
  public : 

    BigIntM(); 
    BigIntM(int);
    BigIntM(unsigned int);
    BigIntM(long); 
    BigIntM(const integer&); // LEDA "integer" type
    BigIntM& operator=(const BigIntM& x) 
    { type = x.type; 
      if (frep != 0) 
        *((integer*)(frep)) = *((integer*)(x.frep)); 
      else frep = x.frep; 
    }
    BigIntM& operator=(int x) { type = BIGINT;
      if (frep != 0) *((integer*)(frep)) = integer(x);
      else { frep = new integer(x); BigIntMAlloc++; } 
    }
    BigIntM& operator=(unsigned int x) { type = BIGINT;
      if (frep != 0) *((integer*)(frep)) = integer(x);
      else {frep = new integer(x); BigIntMAlloc++; } 
    }
    const char* class_name(); 
    integer i_value() const;
    void print();

    friend BigIntM operator + (const BigIntM&, const BigIntM&);
    friend BigIntM operator - (const BigIntM&, const BigIntM&);
    friend BigIntM operator * (const BigIntM&, const BigIntM&);
    friend BigIntM operator / (const BigIntM&, const BigIntM&);

    friend BigIntM operator % (const BigIntM&, const BigIntM&);
    friend BigIntM operator ^ (const BigIntM&, int);

    friend BigIntM operator - (const BigIntM&); // negation

    friend bool operator == (const BigIntM&, const BigIntM&);
    friend bool operator != (const BigIntM&, const BigIntM&);
    friend bool operator  < (const BigIntM&, const BigIntM&);
    friend bool operator <= (const BigIntM&, const BigIntM&);
    friend bool operator  > (const BigIntM&, const BigIntM&);
    friend bool operator >= (const BigIntM&, const BigIntM&);

    friend bool is_zero(const BigIntM&);
    friend int sign(const BigIntM&);
}; 

inline BigIntM::BigIntM() {
  type = BIGINT;
  frep = (void*) (new integer(0)); 
  BigIntMAlloc++; 
  // *((integer*)frep) = integer(0); 
}

inline BigIntM::BigIntM(int a) {
  type = BIGINT;
  frep = (void*) (new integer(a)); 
  BigIntMAlloc++; 
  // *((integer*)frep) = integer(a); 
}

inline BigIntM::BigIntM(unsigned int a) {
  type = BIGINT;
  frep = (void*) (new integer); 
  BigIntMAlloc++; 
  *((integer*)frep) = integer(a); 
}

inline BigIntM::BigIntM(long a) {
  type = BIGINT;
  frep = (void*) (new integer); 
  BigIntMAlloc++; 
  *((integer*)frep) = integer(a); 
}

inline BigIntM::BigIntM(const integer& a) {
  type = BIGINT; 
  frep = (void*)(new integer); 
  BigIntMAlloc++; 
  *((integer*)frep) = a; 
}

inline const char* BigIntM::class_name() {
  return(Class_name); 
}

inline integer BigIntM::i_value() const {
  return(*((integer*)frep));
}

inline void BigIntM::print() {
  Print(*((integer*)frep),cout); 
}

#endif
