/*************************************************************************

  APU R 2.0

  fielddef.h

  (c) 1996, Regents of the University of California. All rights reserved. 

  Author : Ashutosh Rege (rege@cs.berkeley.edu)

*************************************************************************/
  

#if !defined(FIELDDEF_HDR)
#define FIELDDEF_HDR

#include "globals.h"

/* Field elements represent the lowest level of the computation. 
   They are represented as a class which has two members, the
   field type and the representation of the field element which
   is itself a class type. The field element representation class
   is a base class and the actual field element types are derived
   classes from this base class. 
*/

#define EXTRA_ENTRIES 20

#define NOTYPE 0
#define VAR 1
#define SMALLINT 2
#define INTMODP 3
#define BIGINT 4
#define FLOAT 5
#define LEDAFLOAT 6
#define BIGFLOAT 7
#define MIXED 8
#define SLP 9
#define MULTI_INTMODP 10
#define RATFLOATF 11
#define ADFLOAT 12
#define BIGRAT 13

#include "integer.h"
class bigfloat;

class Field_element {
  protected : 
    int type; 
    void* frep; 
    int frep2[EXTRA_ENTRIES]; 
  public : 
    Field_element();
    Field_element(int);
    Field_element(unsigned int);
    Field_element(integer); // LEDA "integer" type
    Field_element(float);
    Field_element(double);
    Field_element(bigfloat);  // LEDA "bigfloat" type
    Field_element(int*); // For "Multi_IntModp" type. 
    int type_of() const; 
    bool is_number() const; 
    void* frep_of() const;
    int* frep2_of(); 
    int* frep2_of() const; 
    const char* class_name() const;
    void print() const;
    int i_value() const; 
    int i_value(int) const; 
    void set_frep2(int*); 

    friend Field_element operator + (const Field_element&,const Field_element&);
    friend Field_element operator - (const Field_element&,const Field_element&);
    friend Field_element operator * (const Field_element&,const Field_element&);
    friend Field_element operator / (const Field_element&,const Field_element&);

    friend Field_element operator - (const Field_element&);

    friend Field_element operator % (const Field_element&,const Field_element&);
    friend Field_element operator ^ (const Field_element&,const Field_element&);
    friend Field_element operator ^ (const Field_element&,int);


    friend bool operator == (const Field_element&,const Field_element&);
    friend bool operator != (const Field_element&,const Field_element&);
    friend bool operator  < (const Field_element&,const Field_element&);
    friend bool operator <= (const Field_element&,const Field_element&);
    friend bool operator  > (const Field_element&,const Field_element&);
    friend bool operator >= (const Field_element&,const Field_element&);

    friend bool is_zero(const Field_element&);

    friend ostream& operator << (ostream&, const Field_element&); 
    /*
    #if MEMORY_MANAGER
      Field_element& operator=(const Field_element&);
    #endif
    */
};

inline Field_element::Field_element() {
  type = NOTYPE;
  frep = 0;
}

inline int Field_element::type_of() const {
  return(type);
}

inline bool Field_element::is_number() const {
  switch(type_of()) {
    case NOTYPE : return(false); break;
    case VAR : return(false); break;
    default : return(true); break;
  }
}

inline void* Field_element::frep_of() const {
  return(frep);
}

/*
inline int* Field_element::frep2_of() {
  return((int*)frep2);
}
*/

inline int* Field_element::frep2_of() const {
  return((int*)frep2);
}

inline void Field_element::set_frep2(int* a) { 
  for (int i=0;i<EXTRA_ENTRIES;i++) 
    frep2[i] = a[i];
}

inline void* frep_of(const Field_element& fe) {
  return(fe.frep_of());
}

/*
inline int* frep2_of(Field_element& fe) {
  return((int*)(fe.frep2_of()));
}
*/

inline int* frep2_of(const Field_element& fe) {
  return(fe.frep2_of());
}

extern const char* Field_element::class_name() const;  
extern void Field_element::print() const;  
extern Field_element make_Field_element(int,int);
extern Field_element make_Field_element(const integer&,int);
extern Field_element make_Field_element(double,int);


#endif
