/*************************************************************************

  APU R 2.0

  primelist.h

  (c) 1996, Regents of the University of California. All rights reserved. 

  Author : Ashutosh Rege (rege@cs.berkeley.edu)

*************************************************************************/
  

#if !defined(PRIMELIST_HDR)
#define PRIMELIST_HDR

#include "globals.h"
#include "array.h"
#include "matrix.h"
#include "fielddef.h"
#include "bigint.h"

#define NUM_PRIMES 1200
extern int finitef_inverse(int,int); 


class PrimeList : public Field_element {
  private : 
  public : 
    int Num_primes;
    Array<unsigned int> primes; 

    Matrix<unsigned int> p_inverses; 
    Array<BigInt> p_prod; 

    PrimeList() {} 
    PrimeList(int); // Constructor with number of primes as input. 
    unsigned int inverses(int,int); // Return the (i,j)^th inverse. 
    BigInt prime_prod(int,bool=false);  // Return the product p_0 ... p_i. 

    int num_primes() { return(Num_primes); } 

    friend void init_primes(); 
}; 

inline PrimeList::PrimeList(int np) { 
  Num_primes = np; 
  primes = Array<unsigned int>(np); 
  p_inverses = Matrix<unsigned int>(np,np); 
  p_prod = Array<BigInt>(np); 
  for (int i=0;i<np;i++) { 
    p_prod[i] = 0; 
    for (int j=0;j<np;j++) 
      p_inverses[i][j] = 0; 
  }
}

inline unsigned int
PrimeList::inverses(int i, int j) { 
  if (p_inverses[i][j] == 0) { 
    p_inverses[i][j] = (unsigned) (finitef_inverse(primes[i],primes[j])); 
  }
  return(p_inverses[i][j]); 
}


inline BigInt
PrimeList::prime_prod(int i, bool free_prev = false) { 
  if (i==0) {
    if (is_zero(p_prod[0])) 
      p_prod[0] = primes[0]; 
    return p_prod[0];
  }
  else if (is_zero(p_prod[i])) { 
    BigInt temp = primes[i]; 
    p_prod[i] = prime_prod(i-1) * temp; 
    // ~(*(integer*)temp.frep_of()); 
    // if (free_prev) 
      // ~(*(integer*)p_prod[i-1].frep_of());
  }
  return p_prod[i]; 
}

extern PrimeList Primes; 

#endif
