/*************************************************************************

  APU R 2.0

  bigfloatpt.h

  (c) 1996, Regents of the University of California. All rights reserved. 

  Author : Ashutosh Rege (rege@cs.berkeley.edu)

*************************************************************************/
  

#if !defined(RATFLOATF_HDR)
#define RATFLOATF_HDR

#include "globals.h"

#include "fielddef.h"
#include "floatftest.h"

class RatFloatf : public Field_element {
  private : 
    static const char* Class_name; 
  public : 

    RatFloatf(); 
    RatFloatf(int);
    RatFloatf(long); 
    RatFloatf(double); 
    RatFloatf(const integer&); // LEDA "integer" type
    RatFloatf(float_filter,float_filter); 
    const char* class_name(); 
    float_filter numer_of() const; 
    float_filter denom_of() const; 

    friend RatFloatf operator + (const RatFloatf&, const RatFloatf&);
    friend RatFloatf operator - (const RatFloatf&, const RatFloatf&);
    friend RatFloatf operator * (const RatFloatf&, const RatFloatf&);
    friend RatFloatf operator / (const RatFloatf&, const RatFloatf&);

    friend RatFloatf operator ^ (const RatFloatf&, int);
    friend RatFloatf operator - (const RatFloatf&); // negation

    friend bool operator  < (const RatFloatf&, const RatFloatf&);
    friend bool operator <= (const RatFloatf&, const RatFloatf&);
    friend bool operator  > (const RatFloatf&, const RatFloatf&);
    friend bool operator >= (const RatFloatf&, const RatFloatf&);

    friend int sign(const RatFloatf&);
}; 

typedef struct rat_floatf_node { 
  float_filter numer; 
  float_filter denom; 
} RatFloatf_node; 

inline RatFloatf::RatFloatf() {
  type = RATFLOATF;
  frep = (void*) (new RatFloatf_node); 
  ((RatFloatf_node*)frep)->numer = float_filter();; 
  ((RatFloatf_node*)frep)->denom = 1; 
}

inline RatFloatf::RatFloatf(int a) {
  type = RATFLOATF;
  frep = (void*) (new RatFloatf_node); 
  ((RatFloatf_node*)frep)->numer = a; 
  ((RatFloatf_node*)frep)->denom = 1; 
}

inline RatFloatf::RatFloatf(long a) {
  type = RATFLOATF;
  frep = (void*) (new RatFloatf_node); 
  ((RatFloatf_node*)frep)->numer = (int)(a); 
  ((RatFloatf_node*)frep)->denom = 1; 
}

inline RatFloatf::RatFloatf(double a) {
  type = RATFLOATF;
  frep = (void*) (new RatFloatf_node); 
  ((RatFloatf_node*)frep)->numer = a; 
  ((RatFloatf_node*)frep)->denom = 1.0; 
}

inline RatFloatf::RatFloatf(const integer& a) {
  type = RATFLOATF;
  frep = (void*) (new RatFloatf_node); 
  ((RatFloatf_node*)frep)->numer = a; 
  ((RatFloatf_node*)frep)->denom = 1; 
}

inline RatFloatf::RatFloatf(float_filter a, float_filter b) {
  type = RATFLOATF;
  frep = (void*) (new RatFloatf_node); 
  ((RatFloatf_node*)frep)->numer = a; 
  ((RatFloatf_node*)frep)->denom = b; 
}
inline const char* RatFloatf::class_name() {
  return(Class_name); 
}

inline float_filter RatFloatf::numer_of() const { 
  return(((RatFloatf_node*)frep)->numer); 
}

inline float_filter RatFloatf::denom_of() const { 
  return(((RatFloatf_node*)frep)->denom); 
}

inline float_filter numer_of(const RatFloatf a) { 
  return(a.numer_of());
}

inline float_filter denom_of(const RatFloatf& a) { 
  return(a.denom_of());
}

#endif
