
#if !defined(SLP_HDR)
#define SLP_HDR

#include "globals.h"
#include "fielddef.h"
#include "list.h"
#include "variable.h"
#include "debug.h"

#define NOP_FE 0 
#define NOP_VAR 1 
#define ADD 2
#define SUB 3
#define MULT 4
#define DIV 5
#define EXP 6
#define DIFF 7

#define slp_node_p(frep) ((Slp_node*)(frep))

struct slp_node; 

class Slp : public Field_element {
  private :
    static const char* Class_name;
    static int Eval_level; 
  public : 
    static List<Variable> var_list;
    static const Slp zero_slp; // global zero slp


    #if MEMORY_MANAGER
    Slp& operator=(const Slp&); 
    #endif
    Slp() { type = SLP; frep = 0; } 
    Slp(int); 
    Slp(unsigned int);
    Slp(float);
    Slp(double);
    Slp(Field_element); 
    Slp(Variable);
    Slp(Variable*);
    Slp(char *); 
    Slp(int,const Slp&, const Slp&);
    int op() const;
    int eval_level() const; 
    void eval_level(int) const;
    Slp left() const;
    Slp right() const;
    Slp anti_deriv_of() const; 
    Array<int> deriv_computed_array() const; 
    Array<Slp> deriv_array() const; 
    Field_element value() const; 
    Field_element fe_of() const;  
    Field_element* fePtr_of() const;
    void set_value(const Field_element&) const;
    bool is_numerical() const;
    bool is_variable() const;
    Variable variable_of() const;


    friend Slp slp_binop(int, const Slp&, const Slp&);
    friend Slp operator + (const Slp&, const Slp&);
    friend Slp operator - (const Slp&, const Slp&);
    friend Slp operator * (const Slp&, const Slp&);
    friend Slp operator / (const Slp&, const Slp&);

    friend Slp operator - (const Slp&);
    friend Slp operator ^ (const Slp&, int);
    friend bool is_zero(const Slp&);

    friend Field_element evaluate(const Slp&,char* = "");

    friend slp_node* frep_of(const Slp&);
    friend DebugClass* debug_info_of(const Slp&);
     
};

/*
inline Slp::Slp() {
  type = SLP;
  frep = 0; 
}
*/

typedef struct slp_node {
  int op; 
  Slp* left; 
  Slp* right;
  int eval_level; 
  Field_element* value; 

  Array<Slp> deriv_array;
  Array<int> deriv_computed; 
  Slp anti_deriv;

  DebugClass* debug_info;
  #if MEMORY_MANAGER
    Mem_mgr mem_info;
  #endif
} Slp_node; 

#if MEMORY_MANAGER
inline Slp& Slp::operator=(const Slp& rvalue) {
  if (rvalue.frep == 0)  {
    if (frep_of() == 0) {
      type = rvalue.type; 
    }
    else {
      ref_count_dec(((Slp_node*)(frep))->mem_info);
      if (ref_count_of(((Slp_node*)(frep))->mem_info) == 0)
	delete frep; 
      type = rvalue.type; 
      frep = rvalue.frep; 
    }
  }
  else {
    ref_count_inc((((Slp_node*)(rvalue.frep))->mem_info));
    if (frep_of() == 0) {
      type = rvalue.type; 
      frep = rvalue.frep; 
    }
    else {
      ref_count_dec(((Slp_node*)(frep))->mem_info);
      if (ref_count_of(((Slp_node*)(frep))->mem_info) == 0)
	delete frep; 
      type = rvalue.type; 
      frep = rvalue.frep; 
      
    }
  }
}
#endif

inline Slp::Slp(int a) {
  type = SLP;
  if (a == 0) frep = 0;
  else {
    frep = (void*) (new Slp_node);
    ((Slp_node*)frep)->op = NOP_FE;
    ((Slp_node*)frep)->left = 0;
    ((Slp_node*)frep)->right = 0;
    ((Slp_node*)frep)->eval_level = 0;
    ((Slp_node*)frep)->value = new Field_element;
    *((Slp_node*)frep)->value = Field_element(a);
    ((Slp_node*)frep)->deriv_array = 0;
    ((Slp_node*)frep)->deriv_computed = 0;

    ((Slp_node*)frep)->debug_info = new DebugClass;
  }
}

inline Slp::Slp(unsigned int a) {
  type = SLP;
  frep = (void*) (new Slp_node);
  ((Slp_node*)frep)->op = NOP_FE;
  ((Slp_node*)frep)->left = 0;
  ((Slp_node*)frep)->right = 0;
  ((Slp_node*)frep)->eval_level = 0;
  ((Slp_node*)frep)->value = new Field_element;
  *((Slp_node*)frep)->value = Field_element(a);
  ((Slp_node*)frep)->deriv_array = 0;
  ((Slp_node*)frep)->deriv_computed = 0;

  ((Slp_node*)frep)->debug_info = new DebugClass;
}

inline Slp::Slp(float a) {
  type = SLP;
  frep = (void*) (new Slp_node);
  ((Slp_node*)frep)->op = NOP_FE;
  ((Slp_node*)frep)->left = 0;
  ((Slp_node*)frep)->right = 0;
  ((Slp_node*)frep)->eval_level = 0;
  ((Slp_node*)frep)->value = new Field_element;
  *((Slp_node*)frep)->value = Field_element(a);
  ((Slp_node*)frep)->deriv_array = 0;
  ((Slp_node*)frep)->deriv_computed = 0;

  ((Slp_node*)frep)->debug_info = new DebugClass;
}

inline Slp::Slp(double a) {
  type = SLP;
  frep = (void*) (new Slp_node);
  ((Slp_node*)frep)->op = NOP_FE;
  ((Slp_node*)frep)->left = 0;
  ((Slp_node*)frep)->right = 0;
  ((Slp_node*)frep)->eval_level = 0;
  ((Slp_node*)frep)->value = new Field_element;
  *((Slp_node*)frep)->value = Field_element(a);
  ((Slp_node*)frep)->deriv_array = 0;
  ((Slp_node*)frep)->deriv_computed = 0;

  ((Slp_node*)frep)->debug_info = new DebugClass;
}

inline Slp::Slp(Field_element a) {
  type = SLP;
  frep = (void*) (new Slp_node);
  ((Slp_node*)frep)->op = NOP_FE;
  ((Slp_node*)frep)->left = 0;
  ((Slp_node*)frep)->right = 0;
  ((Slp_node*)frep)->eval_level = 0;
  ((Slp_node*)frep)->value = new Field_element;
  *((Slp_node*)frep)->value = a; 
  ((Slp_node*)frep)->deriv_array = 0;
  ((Slp_node*)frep)->deriv_computed = 0;

  ((Slp_node*)frep)->debug_info = new DebugClass;
}

inline Slp::Slp(char* str) { 
  Variable* temp;
  temp = var_list.find(str);
  if (temp != 0) 
    *this = *(temp->slpPtr_of()); 
  else 
    cout << "Variable " << str << " not declared as variable\n";
}

inline Slp::Slp(Variable a) {
  type = SLP;
  frep = (void*) (new Slp_node);
  ((Slp_node*)frep)->op = NOP_VAR;
  ((Slp_node*)frep)->left = 0;
  ((Slp_node*)frep)->right = 0;
  ((Slp_node*)frep)->eval_level = 0;
  ((Slp_node*)frep)->value = new Field_element;
  *((Slp_node*)frep)->value = a;
  ((Slp_node*)frep)->deriv_array = 0;
  ((Slp_node*)frep)->deriv_computed = 0;

  ((Slp_node*)frep)->debug_info = new DebugClass;
}

inline Slp::Slp(Variable* a) {
  type = SLP;
  frep = (void*) (new Slp_node);
  ((Slp_node*)frep)->op = NOP_VAR;
  ((Slp_node*)frep)->left = 0;
  ((Slp_node*)frep)->right = 0;
  ((Slp_node*)frep)->eval_level = 0;
  ((Slp_node*)frep)->value = a; 
  ((Slp_node*)frep)->deriv_array = 0;
  ((Slp_node*)frep)->deriv_computed = 0;

  ((Slp_node*)frep)->debug_info = new DebugClass;
}

inline Slp::Slp(int op, const Slp& left, const Slp& right) {
  type = SLP;
  frep = (void*) (new Slp_node);
  ((Slp_node*)frep)->op = op;
  // ((Slp_node*)frep)->left = (Slp*)(&left);
  // ((Slp_node*)frep)->right = (Slp*)(&right);
  ((Slp_node*)frep)->left = new Slp; 
  ((Slp_node*)frep)->right = new Slp; 
  *(((Slp_node*)frep)->left) = left;
  *(((Slp_node*)frep)->right) = right;
  ((Slp_node*)frep)->eval_level = 0;
  ((Slp_node*)frep)->value = 0;
  ((Slp_node*)frep)->deriv_array = 0;
  ((Slp_node*)frep)->deriv_computed = 0;

  ((Slp_node*)frep)->debug_info = new DebugClass;
}

inline int Slp::op() const {
  if (frep == 0) return(NOP_FE); 
  else return(((Slp_node*)frep)->op);
}

inline int Slp::eval_level() const {
  if (frep == 0) return(Eval_level);
  else return(((Slp_node*)frep)->eval_level);
}

inline void Slp::eval_level(int a) const {
  if (frep != 0) ((Slp_node*)frep)->eval_level = a;
}

inline Slp Slp::left() const {
  if (frep == 0) 
    cout << "Slp.left() : Illegal call to Slp - no frep\n";
  else if (((Slp_node*)frep)->left == 0)
    cout << "Slp.left() : Illegal call to Slp - no left child\n";
  else return(*(((Slp_node*)frep)->left));
}

inline Slp Slp::right() const {
  if (frep == 0) 
    cout << "Slp.right() : Illegal call to Slp - no frep\n";
  else if (((Slp_node*)frep)->right == 0)
    cout << "Slp.right() : Illegal call to Slp - no right child\n";
  else return(*(((Slp_node*)frep)->right));
}

inline Slp Slp::anti_deriv_of() const {
  if (frep == 0) 
    cout << "Slp.anti_deriv_of() : Illegal call to Slp - no frep\n";
  else return(((Slp_node*)frep)->anti_deriv);
}


inline Field_element Slp::value() const {
  if (frep == 0) return(Field_element(0));
  else if (((Slp_node*)frep)->value == 0)
    cout << "Slp.value() : Illegal call to Slp - no value node\n";
  else if (op() == NOP_VAR) {
    Slp_node* temp = slp_node_p(frep); 
    Field_element* temp2 = temp->value; 
    return(*(((Variable_node*) (*(temp2)).frep_of() )->value));
  }
  else return(*(((Slp_node*)frep)->value));

  /*
  return(*(((Variable_node*) (*(slp_node_p(frep)->value)).frep_of() )->value));
  return(*(((Variable_node*) (*(((Slp_node*)frep)->value)).frep_of() )->value));
  */
}

inline Field_element Slp::fe_of() const {
  if (frep != 0) {
    if (((Slp_node*)frep)->value != 0)
      return(*(((Slp_node*)frep)->value));
  }
}

inline Field_element* Slp::fePtr_of() const {
  if (frep != 0) {
      return(((Slp_node*)frep)->value);
  }
}

inline void Slp::set_value(const Field_element& fe) const {
  if (frep == 0) 
    cout << "Slp.set_value() : Illegal call to Slp - no frep\n";
  else {
    ((Slp_node*)frep)->value = new Field_element;
    *(((Slp_node*)frep)->value) = fe;
  }
}

inline Slp slp_binop(int op, const Slp& lchild, const Slp& rchild) {
  Slp result = Slp(op,lchild,rchild); 
  return(result);
}

inline Array<int> Slp::deriv_computed_array() const {
  if (frep == 0) 
    cout << "Slp.deriv_computed_array() : Illegal call to Slp - no frep\n";
  else return(((Slp_node*)frep)->deriv_computed);
}

inline Array<Slp> Slp::deriv_array() const {
  if (frep == 0) 
    cout << "Slp.deriv_array() : Illegal call to Slp - no frep\n";
  else return(((Slp_node*)frep)->deriv_array);
}

inline bool Slp::is_numerical() const {
  if (frep != 0) {
    if (op() == NOP_FE) return(true);
    else return(false);
  }
  else return(true);
}

inline bool Slp::is_variable() const {
  if (frep != 0) {
    if (op() == NOP_VAR) return(true);
    else return(false);
  }
  else return(true);
}

inline Variable Slp::variable_of() const {
  if (frep == 0) 
    cout << "Error in variable_of() - Slp has no frep\n";
  else if (op() != NOP_VAR) 
    cout << "Error in variable_of() - called with non-variable Slp\n";
  else return(*((Variable*)(((Slp_node*)frep)->value)));
}

inline Slp_node* frep_of(const Slp& slp) {
  return((Slp_node*)(slp.frep));
}


Slp diff(const Slp&, char*, bool=false); 
Slp slp_partial_derivative(const Slp&, char*, int,bool);
Slp slp_partial_derivative2(const Slp&, char*, int, int,bool);
Slp locate_original(const Slp&, int, int*);
Slp find_partial_derivative(const Slp&, int, int, int*);
Slp compute_slp_single_derivative(const Slp&, char*, int,bool); 
Slp slp_single_terminal_derivative(const Slp&, char*, int,bool);
Slp compute_slp_partial_derivative(const Slp&, char*, int, int,bool);
int binomial_coeff(int,int);
int factorial(int);

#include "slpaux.h"

#endif
