
/*************************************************************************

  APU R 2.0

  slpint.h

  (c) 1996, Regents of the University of California. All rights reserved. 

  Author : Ashutosh Rege (rege@cs.berkeley.edu)

*************************************************************************/
  

#if !defined(SLPINT_HDR)
#define SLPINT_HDR

#include "globals.h"
#include "fielddef.h"
#include "list.h"
#include "variable_slpint.h"
#include "debug.h"
#include "matrix.h"
#include "bigint.h"
#include "intmodp.h"

#define NOP_FE 0 
#define NOP_VAR 1 
#define ADD 2
#define SUB 3
#define MULT 4
#define DIV 5
#define EXP 6
#define DIFF 7

#define slp_node_p(frep) ((SlpInt_node*)(frep))

struct slp_node; 
class Variable_SlpInt; 

class SlpInt : public Field_element {
  private :
    static const char* Class_name;
  public : 
    static int Num_vars; 
    static int Eval_level; 
    static int Degree_eval_level; 
    static List<Variable_SlpInt> var_list; 
    static const SlpInt zero_slp; // global zero slp
    static SlpInt unit_slp; // global SlpInt for 1. 

    const char* class_name(); 
    SlpInt() { type = SLPINT; frep = 0; } 
    SlpInt(int); 
    SlpInt(unsigned int);
    Slp(char *); 
    Slp(int,const SlpInt&, const SlpInt&);
    int op() const;
    int eval_level() const; 
    void eval_level(int) const;
    int degree_eval_level() const; 
    void degree_eval_level(int) const;
    int degree() const; 
    void degree(int) const;
    int deriv_done() const; 
    void deriv_done(int) const;
    SlpInt lchild() const;
    SlpInt rchild() const;
    SlpInt anti_deriv_of() const; 
    Array<int> deriv_computed_array() const; 
    Array<Slp> deriv_array() const; 
    Field_element value() const; 
    Field_element fe_of() const;  
    Field_element* fePtr_of() const;
    void set_value(const Field_element&) const;
    bool is_numerical() const;
    bool is_variable() const;
    Variable variable_of() const;
    slp_node* frep_of() const { return((slp_node*)frep); }


    friend SlpInt slp_binop(int, const SlpInt&, const SlpInt&);
    friend SlpInt operator + (const SlpInt&, const SlpInt&);
    friend SlpInt operator - (const SlpInt&, const SlpInt&);
    friend SlpInt operator * (const SlpInt&, const SlpInt&);
    friend SlpInt operator / (const SlpInt&, const SlpInt&);

    friend SlpInt operator - (const SlpInt&);
    friend SlpInt operator ^ (const SlpInt&, int);
    friend bool is_zero(const SlpInt&);
    friend int sign(const SlpInt&); 

    friend bool operator > (const SlpInt&, const SlpInt&);

    friend SlpInt det(const Matrix<Slp>&); 
    friend Matrix<Slp> gauss_elim(const Matrix<Slp>&); 
    friend SlpInt det2(const Matrix<Slp>&); 
    friend Field_element evaluate(const SlpInt&,char* = "",int* = 0,
			 char* = "", Field_element = 0);
    friend Field_element evaluate_modp(const SlpInt&,char* = "",int* = 0,
			 char* = "", Field_element = 0);
    friend Polynomial 
      interpolate(const SlpInt&, char*, int=INTMODP, char* = "",bool=false);
    friend Polynomial 
      interpolate_int(const SlpInt&, char*, char* = "",bool=false);
    friend Polynomial interpolate_int_numerical(const SlpInt&, char*, 
      Matrix<int>,Matrix<int>);
    friend Array<BigInt> interpolate_int_numerical(Array<Slp>);
    friend int degree_in_var(const SlpInt&, char* ); 
    friend int count_nodes(const SlpInt&); 
    friend Variable* new_slpvariable(char*); 

    /*
    #if MEMORY_MANAGER
    SlpInt& operator=(const SlpInt&); 
    ~Slp(); 
    void clear();
    #endif

    #if DEBUG
    friend DebugClass* debug_info_of(const SlpInt&);
    #endif
    */
};

typedef struct slpint_node {
  int op; 
  SlpInt* left; 
  SlpInt* right;
  int eval_level; 
  int value; 
  char var[5]; 

  int degree; 
  int degree_eval_level; 

} SlpInt_node; 

inline SlpInt::SlpInt(int a) {
  type = SLPINT;
  if (a == 0) frep = 0;
  else if (a == 1) { 
    if (SlpInt::unit_slp.frep == 0) { 
      SlpInt::unit_slp.frep = (void*) (new SlpInt_node);
      ((SlpInt_node*)(SlpInt::unit_slp.frep))->op = NOP_FE;
      ((SlpInt_node*)(SlpInt::unit_slp.frep))->left = 0;
      ((SlpInt_node*)(SlpInt::unit_slp.frep))->right = 0;
      ((SlpInt_node*)(SlpInt::unit_slp.frep))->eval_level = 0;
      ((SlpInt_node*)(SlpInt::unit_slp.frep))->degree_eval_level = 0;
      ((SlpInt_node*)(SlpInt::unit_slp.frep))->degree = 0;
      ((SlpInt_node*)(SlpInt::unit_slp.frep))->value = a; 
    } 
    *this = SlpInt::unit_slp; 
  }
  else {
    frep = (void*) (new SlpInt_node);
    (frep_of())->op = NOP_FE;
    ((SlpInt_node*)frep)->left = 0;
    ((SlpInt_node*)frep)->right = 0;
    ((SlpInt_node*)frep)->eval_level = 0;
    ((SlpInt_node*)frep)->degree_eval_level = 0;
    ((SlpInt_node*)frep)->degree = 0;
    ((SlpInt_node*)frep)->value = a; 
  }
}

inline SlpInt::SlpInt(unsigned int a) {
  type = SLPINT;
  frep = (void*) (new SlpInt_node);
  ((SlpInt_node*)frep)->op = NOP_FE;
  ((SlpInt_node*)frep)->left = 0;
  ((SlpInt_node*)frep)->right = 0;
  ((SlpInt_node*)frep)->eval_level = 0;
  ((SlpInt_node*)frep)->degree_eval_level = 0;
  ((SlpInt_node*)frep)->degree = 0;
  ((SlpInt_node*)frep)->value = a; 

}

inline SlpInt::SlpInt(char* str) { 
  Variable_SlpInt* temp;
  temp = var_list.find(str);
  if (temp != 0) 
    *this = *(temp->slpPtr_of()); 
  else 
    cout << "Variable " << str << " not declared as variable\n";
}

inline SlpInt::Slp(int op, const SlpInt& left, const SlpInt& right) {
  type = SLP;
  frep = (void*) (new SlpInt_node);
  ((SlpInt_node*)frep)->op = op;
  ((SlpInt_node*)frep)->left = new Slp; 
  ((SlpInt_node*)frep)->right = new Slp; 
  *(((SlpInt_node*)frep)->left) = left;
  *(((SlpInt_node*)frep)->right) = right;
  ((SlpInt_node*)frep)->eval_level = 0;
  ((SlpInt_node*)frep)->degree_eval_level = 0;
  ((SlpInt_node*)frep)->degree = 0;
  ((SlpInt_node*)frep)->value = 0;
}

inline slp_node* frep_of(const SlpInt& slp) {
  return(slp.frep_of());
}

inline bool is_zero(const SlpInt& slp) {
  if (slp.frep == 0) return(true); else return(false);
}

inline int SlpInt::op() const {
  if (frep == 0) return(NOP_FE); 
  else return(((SlpInt_node*)frep)->op);
}

inline int SlpInt::eval_level() const {
  if (frep == 0) return(Eval_level);
  else return(((SlpInt_node*)frep)->eval_level);
}

inline void SlpInt::eval_level(int a) const {
  if (frep != 0) ((SlpInt_node*)frep)->eval_level = a;
}

inline int SlpInt::degree_eval_level() const {
  if (frep == 0) return(Degree_eval_level);
  else return(((SlpInt_node*)frep)->degree_eval_level);
}

inline void SlpInt::degree_eval_level(int a) const {
  if (frep != 0) ((SlpInt_node*)frep)->degree_eval_level = a;
}

inline int SlpInt::degree() const {
  if (frep == 0) return(0);
  else return(((SlpInt_node*)frep)->degree);
}

inline void SlpInt::degree(int a) const {
  if (frep != 0) ((SlpInt_node*)frep)->degree = a;
}

inline int SlpInt::deriv_done() const {
  if (frep == 0) return(0);
  else return(((SlpInt_node*)frep)->deriv_done);
}

inline void SlpInt::deriv_done(int a) const {
  if (frep != 0) ((SlpInt_node*)frep)->deriv_done = a;
}

inline SlpInt SlpInt::lchild() const {
  if (frep == 0) 
    cout << "Slp.lchild() : Illegal call to SlpInt - no frep\n";
  else if (((SlpInt_node*)frep)->left == 0)
    cout << "Slp.lchild() : Illegal call to SlpInt - no left child\n";
  else return(*(((SlpInt_node*)frep)->left));
}

inline SlpInt SlpInt::rchild() const {
  if (frep == 0) 
    cout << "Slp.rchild() : Illegal call to SlpInt - no frep\n";
  else if (((SlpInt_node*)frep)->right == 0)
    cout << "Slp.rchild() : Illegal call to SlpInt - no right child\n";
  else return(*(((SlpInt_node*)frep)->right));
}

inline int SlpInt::value() const {
  if (frep == 0) return(0); 
  else return(*(((SlpInt_node*)frep)->value));
}

inline void SlpInt::set_value(int val) const {
  if (frep == 0) 
    cout << "Slp.set_value() : Illegal call to SlpInt - no frep\n";
  else (((SlpInt_node*)frep)->value) = val;
}

inline SlpInt slp_binop(int op, const SlpInt& lchild, const SlpInt& rchild) {
  SlpInt result = SlpInt(op,lchild,rchild); 
  return(result);
}

inline bool SlpInt::is_numerical() const {
  if (frep != 0) {
    if (op() == NOP_FE) return(true);
    else return(false);
  }
  else return(true);
}

inline bool SlpInt::is_variable() const {
  if (frep != 0) {
    if (op() == NOP_VAR) return(true);
    else return(false);
  }
  else return(true);
}

inline Variable SlpInt::variable_of() const {
  if (frep == 0) 
    cout << "Error in variable_of() - SlpInt has no frep\n";
  else if (op() != NOP_VAR) 
    cout << "Error in variable_of() - called with non-variable Slp\n";
  else return(*((Variable_SlpInt*)(((SlpInt_node*)frep)->value)));
}

inline const char* SlpInt::class_name() {
  return(Class_name); 
}

inline Variable_SlpInt* new_slpintvariable(char* var) { 
  Variable_SlpInt* temp = new Variable_SlpInt(var); 
  temp->index((SlpInt::Num_vars)); 
  temp->slp(SlpInt(temp)); 
  temp->value(((int)(lrand48() % IntModp::prime)));
  SlpInt::var_list.add(*temp); 
  (SlpInt::Num_vars)++; 
  return temp; 
}

#endif
