
/*************************************************************************

  APU R 2.0

  convert.c

  (c) 1996, Regents of the University of California. All rights reserved. 

  Author : Ashutosh Rege (rege@cs.berkeley.edu)

*************************************************************************/
  
#include <fstream.h>
#include <ctype.h>
#include "polynomial.h"
#include "molynomial.h"
#include "util.h"
#include "slp.h"
#include "integer.h"


Slp poly_to_slp(Polynomial); 

Slp
poly_to_slp (Polynomial p)
{
   Slp result;
   Field_element fe;
   int temp_int1,temp_int2; double temp_float; 
   char c; char* var; 

   if (is_terminal(p)) { 
     result = Slp(fe_of(p)); 
   }
   else { // Polynomial with coeffs which are polynomials which means
     Slp coeff,var_slp;
     int deg, done; Variable* temp_v; 
     var = (variable_of(p))->name();  
     temp_v = Slp::var_list.find(var); // check if variable is legal
     if (temp_v == 0) {
       cout << "Error in poly_to_slp() - non Slp variable\n"; 
       return(Slp(0)); 
     }
     TermNode* tn; 
     tn = (p.termlist_of()).tnode_of(); 
     int prev_d = tn->degree_of(); 
     result = poly_to_slp(tn->coeff_poly()); 
     Slp temp_slp; 
     var_slp = Slp(var); 
     for (tn=tn->next_of();tn!=0;tn=tn->next_of()) { 
       temp_slp = var_slp ^ (prev_d - tn->degree_of()); 
       result = result * temp_slp; 
       result = result + poly_to_slp(tn->coeff_poly()); 
       prev_d = tn->degree_of();
     }
     if (prev_d > 0) { 
       temp_slp = var_slp ^ prev_d; 
       result = result*temp_slp; 
     }
  }
  return result; 
}

Molynomial
poly_to_moly(const Polynomial& poly) 
{
  if (is_terminal(poly)) { // Create a monomial with expons all 0 and 
			   // coeff equal to field elt of poly
    Monomial temp_m = fe_of(poly);  
    return(Molynomial(temp_m)); 
  }
  int ind = (variable_of(poly))->index(); 
  TermList tl = termlist_of(poly); 
  TermNode* tn;
  List<Monomial> temp_monoms;
  Molynomial temp,result; 
  for (tn=tnode_of(tl);tn!=0;tn=tn->next_of()) { 
     temp = poly_to_moly(tn->coeff_poly()); 
     temp_monoms = monoms_of(temp); 
     ListNode<Monomial>* mptr; 
     for (mptr = head_of(temp_monoms);mptr !=0;mptr=mptr->next_of()) { 
       Monomial temp_m = mptr->item_of(); 
       temp_m.expons[ind] = tn->degree_of(); 
     }
     result = concat(result,temp); 
  }
  return result;
}
