
/*************************************************************************

  APU R 2.0

  poly_arith.c

  (c) 1996, Regents of the University of California. All rights reserved. 

  Author : Ashutosh Rege (rege@cs.berkeley.edu)

*************************************************************************/
  
#include "polynomial.h"

const char* Polynomial::Class_name = "Polynomial"; 
extern int Debug;

Polynomial
operator + (const Polynomial& a, const Polynomial& b) {

  Polynomial result; 
  TermList temp; 

  if (is_zeropoly(a)) 
    result = b; 
  else if (is_zeropoly(b))
    result = a;
  else if (is_terminal(a)) { 
    if (is_terminal(b)) { 
      Field_element temp_fe; 
      temp_fe = fe_of(a) + fe_of(b); 
      if (type_of(temp_fe) == SLP) { 
	if (is_zero(evaluate_modp((Slp&)temp_fe)))
	  result = Polynomial::zero_poly; 
        else result = Polynomial(temp_fe); 
      }
      else result = Polynomial(temp_fe); 
    }
    else {
      temp = fe_of(a) + termlist_of(b); 
      result = Polynomial(variable_of(b),temp); 
    }
  }
  else if (is_terminal(b)) {
    temp = termlist_of(a) + fe_of(b); 
    result = Polynomial(variable_of(a),temp); 
  }
  else if (variable_of(a)->index() == variable_of(b)->index()) {
    temp = termlist_of(a) + termlist_of(b); 
    result = Polynomial(variable_of(a),temp); 
  }
  else if (variable_of(a)->index() < variable_of(b)->index()) {
    result = Polynomial(variable_of(a), termlist_of(a) + b);  
  }
  else if (variable_of(a)->index() > variable_of(b)->index()) {
    result = Polynomial(variable_of(b), a + termlist_of(b)); 
  }
  return(result);
}

/* Note :  the operator + (fe,poly) is defined inline in the include file 
   as the following + operator with arguments switched */
Polynomial
operator + (const Polynomial& poly, const Field_element& fe) {
  if (is_zeropoly(poly)) {
    return(Polynomial(fe));
  }
  else if (is_terminal(poly)) { 
    return(Polynomial(fe_of(poly) + fe));
  }
  else return(Polynomial(variable_of(poly),termlist_of(poly) + fe));
}


Polynomial
operator - (const Polynomial& a, const Polynomial& b) { 

  Polynomial result; 
  TermList temp; 

  if (is_zeropoly(a)) 
    result = - b; 
  else if (is_zeropoly(b))
    result = a; 
  else if (is_terminal(a)) { 
    if (is_terminal(b)) { 
      Field_element temp_fe; 
      temp_fe = fe_of(a) - fe_of(b); 
      if (type_of(temp_fe) == SLP) { 
	if (is_zero(evaluate_modp((Slp&)temp_fe)))
	  result = Polynomial::zero_poly; 
        else result = Polynomial(temp_fe); 
      }
      else result = Polynomial(temp_fe); 
    }
    else {
      temp = fe_of(a) - termlist_of(b); 
      result = Polynomial(variable_of(b),temp); 
    }
  }
  else if (is_terminal(b)) {
    temp = termlist_of(a) - fe_of(b); 
    result = Polynomial(variable_of(a),temp); 
  }
  else if (variable_of(a)->index() == variable_of(b)->index()) {
    temp = termlist_of(a) - termlist_of(b); 
    result = Polynomial(variable_of(a),temp); 
  }
  else if (variable_of(a)->index() < variable_of(b)->index()) {
    result = Polynomial(variable_of(a), termlist_of(a) - b);  
  }
  else if (variable_of(a)->index() > variable_of(b)->index()) {
    result = Polynomial(variable_of(b), a - termlist_of(b)); 
  }
  if (Debug) 
    cout << result << "\n";
  return(result);
}

Polynomial
operator - (const Polynomial& poly, const Field_element& fe) {
  if (is_zeropoly(poly)) {
    return(Polynomial(- fe));
  }
  else if (is_terminal(poly)) { 
    return(Polynomial(fe_of(poly) - fe));
  }
  else return(Polynomial(variable_of(poly),termlist_of(poly) - fe));
}

Polynomial
operator - (const Field_element& fe, const Polynomial& poly) {
  if (is_zeropoly(poly)) {
    return(Polynomial(fe));
  }
  else if (is_terminal(poly)) { 
    return(Polynomial(fe - fe_of(poly)));
  }
  else return(Polynomial(variable_of(poly),fe - termlist_of(poly)));
}

/* Unary minus */
Polynomial
operator - (const Polynomial& poly) {

  if (is_zeropoly(poly)) 
    return(Polynomial::zero_poly);

  Polynomial result; 
  Polynomial temp = Polynomial(-1);
  result = temp*poly; 
  return(result); 
}

Polynomial 
operator * (const Polynomial& a, const Polynomial& b) {
  Polynomial result; 

  if ((is_zeropoly(a)) || (is_zeropoly(b)))
    result = Polynomial::zero_poly;
  else if (is_terminal(a)) { 
    if (is_terminal(b)) { 
      Field_element temp_fe; 
      temp_fe = fe_of(a) * fe_of(b); 
      if (type_of(temp_fe) == SLP) { 
	if (is_zero(evaluate_modp((Slp&)temp_fe)))
	  result = Polynomial::zero_poly; 
        else result = Polynomial(temp_fe); 
      }
      else result = Polynomial(temp_fe); 
    }
    else 
      result = Polynomial(variable_of(b), fe_of(a) * termlist_of(b));
  }
  else if (is_terminal(b)) 
    result = Polynomial(variable_of(a), termlist_of(a) * fe_of(b));

  else if (variable_of(a)->index() == variable_of(b)->index()) 
    result = Polynomial(variable_of(a), termlist_of(a) * termlist_of(b));

  else if (variable_of(a)->index() < variable_of(b)->index()) 
    result = Polynomial(variable_of(a), termlist_of(a) * b);

  else if (variable_of(a)->index() > variable_of(b)->index()) 
    result = Polynomial(variable_of(b), a * termlist_of(b));

  if (Debug) cout << result << "\n";
  return(result);
}

/* Note :  the operator * (fe,poly) is defined inline in the include file 
   as the following * operator with arguments switched */
Polynomial
operator * (const Polynomial& poly, const Field_element& fe) {
  if (is_zeropoly(poly)) {
    return(Polynomial::zero_poly);
  }
  else if (is_terminal(poly)) { 
    return(Polynomial(fe_of(poly) * fe));
  }
  else return(Polynomial(variable_of(poly),termlist_of(poly) * fe));
}

Polynomial 
operator / (const Polynomial& a, const Polynomial& b) {
  Polynomial result; 

  if (is_zeropoly(a)) 
    result = Polynomial::zero_poly;
  else if (is_zeropoly(b)) 
    cout << "Divide by zero polynomial in / \n";
  else if (is_terminal(a)) { 
    if (is_terminal(b)) { 
      Field_element temp_fe; 
      temp_fe = fe_of(a) / fe_of(b); 
      if (type_of(temp_fe) == SLP) { 
	if (is_zero(evaluate_modp((Slp&)temp_fe)))
	  result = Polynomial::zero_poly; 
        else result = Polynomial(temp_fe); 
      }
      else result = Polynomial(temp_fe); 
    }
    else 
      result = Polynomial(variable_of(b), fe_of(a) / termlist_of(b));
      // Note that it is possible for a poly to be non-terminal i.e.
      // defined in say "x" with degree 0 and coeff some number. 
      // Hence we don't return 0 here - division of a number by an
      // actual polynomial will be detected in the termlist division.
  }
  else if (is_terminal(b)) 
    result = Polynomial(variable_of(a), termlist_of(a) / fe_of(b));

  else if (variable_of(a)->index() == variable_of(b)->index()) 
    result = Polynomial(variable_of(a), termlist_of(a) / termlist_of(b));

  else if (variable_of(a)->index() < variable_of(b)->index()) 
    result = Polynomial(variable_of(a), termlist_of(a) / b);

  else if (variable_of(a)->index() > variable_of(b)->index()) 
    result = Polynomial(variable_of(b), a / termlist_of(b));
    // Note again that it's possible for the termlist of b to 
    // be made up only of a 0 degree term and a coeff which 
    // could well divide a. This is checked in the termlist division. 

  return(result);
}

Polynomial 
operator / (const Polynomial& poly, const Field_element& fe) {
  if (is_zeropoly(poly)) {
    return(Polynomial::zero_poly);
  }
  else if (is_terminal(poly)) { 
    return(Polynomial(fe_of(poly) / fe));
  }
  else return(Polynomial(variable_of(poly),termlist_of(poly) / fe));
}

Polynomial
operator / (const Field_element& fe, const Polynomial& poly) {
  if (is_zeropoly(poly)) {
    cout << "Divide by zero polynomial in / \n";
  }
  else if (is_terminal(poly)) { 
    return(Polynomial(fe / fe_of(poly)));
  }
  else return(Polynomial(variable_of(poly),fe / termlist_of(poly)));
}

Polynomial
operator ^ (const Polynomial& poly, int expon) {
  Polynomial result;
  if (expon == 0) 
    result = Polynomial(1);
  else {
    result = poly;
    for (int i=0;i<expon-1;i++) 
      result = result*poly;
  }
  return(result);
}

Polynomial
operator % (const Polynomial& a, const Polynomial& b) {
  Polynomial result; 

  if (is_zeropoly(a)) 
    result = Polynomial::zero_poly;
  else if (is_zeropoly(b)) 
    cout << "Error in % (Polynomial,Polynomial) - zero divisor\n";
  else if (is_terminal(a)) { 
    if (is_terminal(b)) 
      result = Polynomial(fe_of(a) % fe_of(b)); 
    else 
      result = Polynomial(variable_of(b), fe_of(a) % termlist_of(b));
      // Note that it is possible for a poly to be non-terminal i.e.
      // defined in say "x" with degree 0 and coeff some number. 
      // Hence we don't return 'a' here - remainder of a number by an
      // actual polynomial will be detected in the termlist division.
  }
  else if (is_terminal(b)) 
    result = Polynomial(variable_of(a), termlist_of(a) % fe_of(b));

  else if (variable_of(a)->index() == variable_of(b)->index()) 
    result = Polynomial(variable_of(a), termlist_of(a) % termlist_of(b));

  else if (variable_of(a)->index() < variable_of(b)->index()) 
    result = Polynomial(variable_of(a), termlist_of(a) % b);

  else if (variable_of(a)->index() > variable_of(b)->index()) 
    result = Polynomial(variable_of(b), a % termlist_of(b));
    // Note again that it's possible for the termlist of b to 
    // be made up only of a 0 degree term and a coeff which 
    // could well divide a. This is checked in the termlist division. 

  return(result);
}

Polynomial 
operator % (const Polynomial& poly, const Field_element& fe) {
  if (is_zeropoly(poly))
    return(Polynomial::zero_poly);
  else if (is_zero(fe))
    cout << "Error in % (Polynomial,Field_element) - zero divisor\n";
  else if (is_terminal(poly)) { 
    return(Polynomial(fe_of(poly) % fe));
  }
  else return(Polynomial(variable_of(poly),termlist_of(poly) % fe));
}

Polynomial
operator % (const Field_element& fe, const Polynomial& poly) {
  if (is_zero(fe))
    return(Polynomial::zero_poly);
  else if (is_zeropoly(poly))
    cout << "Error in % (Field_element,Polynomial) - zero divisor\n";
  else if (is_terminal(poly)) { 
    return(Polynomial(fe % fe_of(poly)));
  }
  else return(Polynomial(variable_of(poly),fe % termlist_of(poly)));
}

