
/*************************************************************************

  APU R 2.0

  poly_interp.c

  (c) 1996, Regents of the University of California. All rights reserved. 

  Author : Ashutosh Rege (rege@cs.berkeley.edu)

*************************************************************************/
  
#include "poly_interp.h"
#include "array.h"

extern int Debug;

Polynomial
vanderpolate(Array<Polynomial> vals_array, Array<Field_element> eval_points,
	     int n, char* var, int poly_type = INTMODP) 
{
  Array<Polynomial> master_coeffs = 
	  get_masterpoly_coeffs(n,poly_type,eval_points); 
  Polynomial master_poly = get_master_poly(master_coeffs,n,var);
  Polynomial divisor_i, Qi, Pi; 
  Field_element Qi_eval;
  Array<Field_element> coeff_array = Array<Field_element>(n+1); 
  for (int i=0;i<=n;i++) {
    cout << sbrk(0) << "\n"; 
    divisor_i = get_ith_divisor(var,eval_points[i],poly_type); 
    if (Debug) 
      cout << master_poly << "\n"; 
    if (Debug) 
      cout << divisor_i << "\n"; 
    cout << sbrk(0) << "\n"; 
    Qi = master_poly/divisor_i; 
    cout << sbrk(0) << "\n"; 
    if (Debug) 
      cout << Qi << "\n"; 
    Qi_eval = get_Qi_eval(i, eval_points, n, poly_type);  
    cout << sbrk(0) << "\n"; 
    if (Debug) 
      cout << Qi_eval << "\n"; 
    // Pi = Qi/Qi_eval;
    update_coeffs(coeff_array,Qi,Qi_eval,vals_array,i,n,var,poly_type); 
    cout << sbrk(0) << "\n"; 
    if (Debug) 
      cout << coeff_array;

  }
  Polynomial result = make_interpolated_poly(coeff_array,n,var,poly_type); 
  return result;
}

Array<Polynomial>
get_masterpoly_coeffs(int n,int poly_type,Array<Field_element> eval_points) 
{
  Array<Polynomial> result = Array<Polynomial>(n+2); 
  cout << sbrk(0) << "\n"; 
  Array<Field_element> mp_coeffs = get_mp_coeffs(n,poly_type,eval_points); 
  cout << sbrk(0) << "\n"; 
  for (int i=0;i<n+2;i++) 
    result[i] = Polynomial(mp_coeffs[i]);
  cout << sbrk(0) << "\n"; 
  return result;
}

Array<Field_element> 
get_mp_coeffs(int n,int poly_type, Array<Field_element> eval_points)
{
  int local_poly_type; 

  if (poly_type == BIGINT) local_poly_type = BIGRAT; 
  else local_poly_type = poly_type;  

  cout << sbrk(0) << "\n"; 
  Array<Field_element> result = Array<Field_element>(n+2);
  cout << sbrk(0) << "\n"; 
  result[0] = make_Field_element(1,local_poly_type); 
  result[1] = - eval_points[0]; 
  for (int i=2;i<n+2;i++) 
    result[i] = make_Field_element(0,local_poly_type); 
  Field_element temp, temp2; 
  for (int i=1;i<n+1;i++) {
    temp2 = result[0];
    for (int j=1;j<=i+1;j++) {
      temp = - eval_points[i]*temp2 + result[j];
      temp2 = result[j];
      result[j] = temp;
    }
  cout << sbrk(0) << "\n"; 
  }
  return result;
}

Polynomial
get_master_poly(Array<Polynomial> master_coeffs, int n, char* var) 
{
  TermList tl = TermNode(n+1,master_coeffs[0]); 
  TermNode* tnodeP = tl.last(); 
  Polynomial result = Polynomial(var,tl);  
  for (int i=1;i<n+2;i++) {
    tnodeP->set_next(TermNode(n+1-i,master_coeffs[i])); 
    tnodeP = tnodeP->next_of();
  }
  return result;
}

/* Returns the poly Z - k_i where var = "Z"*/
Polynomial
get_ith_divisor(char* var, Field_element fe, int poly_type) 
{
  int local_poly_type; 
  if (poly_type == BIGINT) local_poly_type = BIGRAT; 
  else local_poly_type = poly_type; 
  Field_element temp = make_Field_element(1,local_poly_type);
  TermList tl = TermList(TermNode(1,Polynomial(temp))); 
  TermNode* tnodeP = tl.last();
  tnodeP->set_next(TermNode(0,Polynomial(-fe)));
  Polynomial result = Polynomial(var,tl); 
  return result;
}

Field_element
get_Qi_eval(int i, Array<Field_element> eval_points, int n, int poly_type)
{
  int local_poly_type; 
  if (poly_type == BIGINT) local_poly_type = BIGRAT; 
  else local_poly_type = poly_type; 

  Field_element result = make_Field_element(1,local_poly_type); 
  for (int j=0;j<=n;j++) {
    if (j != i) 
      result = result * (eval_points[i] - eval_points[j]); 
  }
  return result;
}

void
update_coeffs(Array<Field_element> coeffs, Polynomial Qi, Field_element Qi_eval,
     Array<Polynomial> vals_array, int i, int n, char* var, int poly_type)
{
  TermList tl = termlist_of(Qi); 
  TermNode* t = tl.tnode_of(); 
  Field_element temp; 
  for (int j=n;j>=0;j--) {
    if (t->degree_of() == j) {
      if (i==0) { 
	temp = (vals_array[i]).fe_of()*(t->coeff_poly()).fe_of(); 
	coeffs[j] = temp/Qi_eval; 
      }
      else {
	temp = (vals_array[i]).fe_of()*(t->coeff_poly()).fe_of(); 
	coeffs[j]  = coeffs[j] + temp/Qi_eval; 
      }
      t = t->next_of();
    }
  }
}

Polynomial
make_interpolated_poly(Array<Field_element> coeffs, int n, char* var, int type)
{

  int j;
  j = n; 
  while((is_zero(coeffs[j])) && (j >= 0)) {
    j--;
  }
  if (j < 0) return(Polynomial::zero_poly); 

  TermList tl = TermList(TermNode(j,Polynomial(coeffs[j])));
  Polynomial result = Polynomial(var,tl);
  TermNode* tnodeP = tl.last(); 
  for (int i=j-1;i>=0;i--) {
    if (!is_zero(coeffs[i])) {
      tnodeP->set_next(TermNode(i,Polynomial(coeffs[i]))); 
      tnodeP = tnodeP->next_of();
    }
  }
  return result;
}
