
/*************************************************************************

  APU R 2.0

  poly_io.c

  (c) 1996, Regents of the University of California. All rights reserved. 

  Author : Ashutosh Rege (rege@cs.berkeley.edu)

*************************************************************************/
  
#include <fstream.h>
#include <ctype.h>
#include "polynomial.h"
#include "molynomial.h"
#include "util.h"
#include "slp.h"
#include "integer.h"

Slp read_polyToSlp (ifstream&, int, int); 

void 
return_error(char* s1, char* s2 = "");

/* The flag "readslp" indicates whether Slp variables, when encountered
   should be read as Slp's or as regular variables */

Polynomial
read_poly(ifstream& inFile, int input_type = SMALLINT, 
	  int poly_type = INTMODP, int slp_type = INTMODP, bool readslp=true)
{
   if ((input_type != SMALLINT) && (input_type != FLOAT) && (input_type !=
	BIGINT))
     return_error("Error in input file", "Input type not SMALLINT or FLOAT");
     // will cause program to exit

   Polynomial result;
   Field_element fe;
   int temp_int1,temp_int2; double temp_float; 
   integer temp_bigint1; 
   char c,var[10]; 
   eat_whitespace(inFile); 
   inFile.get(c); eat_whitespace(inFile); 
   if (c != '(') 
     return_error("Polynomial syntax error in input file"); 
     // will cause program to exit
   inFile.get(c); 
   if (isdigit(c) || c == '-' || c == '.') { // number encountered
     inFile.putback(c);
     switch(input_type) {
       case SMALLINT : 
	 inFile >> temp_int1; 
	 fe = make_Field_element(temp_int1,poly_type); 
	 break;
       case FLOAT : 
	 inFile >> temp_float; 
	 fe = make_Field_element(temp_float,poly_type); 
	 break;
       case BIGINT : 
	 inFile >> temp_bigint1;
	 fe = make_Field_element(temp_bigint1,poly_type); 
     }
     eat_whitespace(inFile); 
     inFile.get(c);
     if (c != ')') 
       return_error("Error in input file");
     else {
       result = Polynomial(fe);
     }
   }
   else if (c == 'F') { // Maple sometimes uses the format Float(int,exp) for
			// representing floating point numbers - this 
			// takes care of that possibility.
     while (c != '(') 
       inFile.get(c); 
     inFile >> temp_int1; 
     inFile.get(c); 
     inFile >> temp_int2; 
     temp_float = (double)(temp_int1)*pow(10.0,((double)(temp_int2)));
     eat_whitespace(inFile); 
     inFile.get(c); 
     if (c != ')')
       return_error("Error in input file");
     fe = make_Field_element(temp_float,poly_type);
     result = Polynomial(fe);
   }
   else { // Polynomial with coeffs which are polynomials which means
	  // it's of the form (var deg1 (...) deg2 (...) ... degN (...))
     int local_poly_type = poly_type; 
     TermList tl;
     Polynomial coeff;
     int deg, done; Variable* temp_v; 
     inFile.putback(c); 
     inFile >> var; 
     temp_v = 0; 
     if ((var[0] == 's') || (var[0] == '_')) { 
       if (readslp) { // means Slp variables should be read in as Slp's 
         temp_v = Slp::var_list.find(var); // check if Slp variable is declared
         if (temp_v == 0) { // means the var was not found so create it.
	   Variable* temp = new_slpvariable(var); 
	   temp_v = temp; 
         }
       }
     } 
     if (temp_v == 0) {
       temp_v = Polynomial::var_list.find(var); // check if variable is legal
       if (temp_v == 0) { 
	 Variable* temp = new_polyvariable(var); 
	 temp_v = temp; 
       }
     }
     else {
       // Read the rest of the poly as an Slp and return Poly of Slp as result 
       for (int i=strlen(var)-1;i>=0;i--) 
	 inFile.putback(var[i]);
       inFile.putback('(');
       result = Polynomial(read_polyToSlp(inFile,input_type,slp_type));
       return(result);
     }
     eat_whitespace(inFile); 
     inFile >> deg; 
     eat_whitespace(inFile); 
     inFile.get(c);
     if (c != '(') 
       return_error("Error in input file");

     inFile.putback(c);
     switch(local_poly_type) {
       case SLP : 
	 coeff = Polynomial(read_polyToSlp(inFile, input_type,slp_type)); 
	 break;
       default : 
	 coeff = read_poly(inFile,input_type,poly_type,slp_type,readslp);
     }
     tl = TermList(TermNode(deg,coeff));
     TermNode* tnodeP = tl.last();
     result = Polynomial(var,tl);
     switch(local_poly_type) {
       case SLP : 
	 done = 0; 
	 while (!done) {
           eat_whitespace(inFile); 
	   inFile.get(c); 
	   if (isdigit(c)) {
	     inFile.putback(c); 
	     inFile >> deg; 
	     eat_whitespace(inFile); 
	     inFile.get(c); 
	     if (c != '(')
	       return_error("Error in input file");
	     inFile.putback(c);
	     coeff = Polynomial(read_polyToSlp(inFile,input_type,slp_type)); 
	     tnodeP->set_next(TermNode(deg,coeff));
	     tnodeP = tnodeP->next_of();
	   }
	   else if (c == ')') 
	     done = 1;
	   else return_error("Error in input file");  
	 }
	 break;
       default : 
	 done = 0; 
	 while (!done) {
           eat_whitespace(inFile); 
	   inFile.get(c); 
	   if (isdigit(c)) {
	     inFile.putback(c); 
	     inFile >> deg; 
	     eat_whitespace(inFile); 
	     inFile.get(c); 
	     if (c != '(')
	       return_error("Error in input file");
	     inFile.putback(c);
	     coeff = read_poly(inFile,input_type,poly_type,slp_type,readslp); 
	     tnodeP->set_next(TermNode(deg,coeff));
	     tnodeP = tnodeP->next_of();
	   }
	   else if (c == ')') 
	     done = 1;
	   else return_error("Error in input file");  
	 }
	 break;
     }
   }
   return(result);
}

void 
return_error(char* s1, char* s2 = "") {
  cout << s1 << ' ' << s2 << "\n"; 
  exit(1);
}

istream&
operator >> (istream& is, Polynomial& poly) {
   poly = read_poly((ifstream&)is);
}

Slp
read_polyToSlp (ifstream& inFile, int input_type, int slp_type) 
{
   Slp result;
   Field_element fe;
   int temp_int1,temp_int2; double temp_float; 
   char c,var[10]; 
   eat_whitespace(inFile); 
   inFile.get(c); eat_whitespace(inFile); 
   if (c != '(') 
     return_error("Polynomial syntax error in input file"); 
     // will cause program to exit
   inFile.get(c); 
   if (isdigit(c) || c == '-' || c == '.') { // number encountered
     inFile.putback(c);
     switch(input_type) {
       case SMALLINT : 
	 inFile >> temp_int1; 
	 if (slp_type == SMALLINT)  
	   result = Slp(temp_int1);
	 else 
	   result = Slp(make_Field_element(temp_int1,slp_type));
	 break;
       case FLOAT : 
	 inFile >> temp_float; 
	 if (slp_type == FLOAT)  
	   result = Slp(temp_float);
	 else 
	   result = Slp(make_Field_element(temp_float,slp_type));
	 break;
     }
     eat_whitespace(inFile); 
     inFile.get(c);
     if (c != ')') 
       return_error("Error in input file");
   }
   else if (c == 'F') { // Maple sometimes uses the format Float(int,exp) for
			// representing floating point numbers - this 
			// takes care of that possibility.
     while (c != '(') 
       inFile.get(c); 
     inFile >> temp_int1; 
     inFile.get(c); 
     inFile >> temp_int2; 
     temp_float = (double)(temp_int1)*pow(10.0,((double)(temp_int2)));
     eat_whitespace(inFile); 
     inFile.get(c); 
     if (c != ')')
       return_error("Error in input file");
     if (slp_type == FLOAT) 
       result = Slp(temp_float);
     else result = Slp(make_Field_element(temp_float,slp_type));
   }
   else { // Polynomial with coeffs which are polynomials which means
	  // it's of the form (var deg1 (...) deg2 (...) ... degN (...))
     Slp coeff,var_slp;
     int deg, done; Variable* temp_v; 
     inFile.putback(c); 
     inFile >> var; 
     temp_v = Slp::var_list.find(var); // check if variable is legal
     if (temp_v == 0) { // means the var was not found so create it.
       Variable* temp = new_slpvariable(var); 
       temp_v = temp; 
     }
     eat_whitespace(inFile); 
     inFile >> deg; 
     var_slp = temp_v->slp_of();
     eat_whitespace(inFile); 
     inFile.get(c);
     if (c != '(') 
       return_error("Error in input file");

     inFile.putback(c);

     result = read_polyToSlp(inFile,input_type,slp_type); 
     int prev_deg = deg;  
     Slp temp_slp; 

     done = 0; 
     while (!done) {
       eat_whitespace(inFile); 
       inFile.get(c); 
       if (isdigit(c)) {
         inFile.putback(c); 
         inFile >> deg; 
         eat_whitespace(inFile); 
         inFile.get(c); 
         if (c != '(')
           return_error("Error in input file");
	 inFile.putback(c);
	 temp_slp = var_slp ^ (prev_deg-deg); 
	 result = result * temp_slp; 
	 result = result + read_polyToSlp(inFile,input_type,slp_type);
	 prev_deg = deg; 
       }
       else if (c == ')') 
         done = 1;
       else return_error("Error in input file");  
     }
     if (prev_deg > 0) { 
       temp_slp = var_slp ^ prev_deg; 
       result = result*temp_slp; 
     }
   }
   return(result);
}

ostream&
operator << (ostream& os, const Polynomial& poly) {
  poly.print(os,0); 
  return os;
}

void
Polynomial::print (ostream& os, int disp) const {
  if (is_zeropoly()) {
    os << "(0)"; 
  }
  else if (is_terminal()) {
    os << "(" << fe_of() << ")"; 
  }
  else {
    os << "(" << fe_of() << " "; 
    (termlist_of()).print(os, disp + variable_of()->index() + 
		      strlen(variable_of()->name())); 
    for (int i=0;i<variable_of()->index();i++) 
      os << " "; 
    os << ")";
  }
}
 
ostream& 
operator << (ostream& os, const Molynomial& moly) { 
  moly.print(os); 
  return os; 
}

void
Molynomial::print(ostream& os) const { 
  List<Monomial> mlist; 
  ListNode<Monomial>* temp; 
  mlist = monoms_of(); 
  for (temp=head_of(mlist);temp!=0;temp=temp->next_of()) { 
    Monomial temp_m = temp->item_of(); 
    os << *(fePtr_of(temp_m));
    int num_vars = size_of(expons_of(temp_m));
    for (int i=0;i<num_vars;i++) { 
      if (expons_of(temp_m)[i] > 0) { 
        Variable* temp = (Polynomial::var_list).find(i); 
        os << "*" << *temp << "^" << expons_of(temp_m)[i];
      }
    }
    if (temp->next_of() != 0) 
        os << " +"; 
    os << "\n"; 
  }
}

List<Polynomial>
read_polylist(ifstream& inFile, int n, int input_type = SMALLINT, 
  int poly_type = INTMODP, int slp_type = INTMODP, bool readslp=true)
{
  Polynomial temp; 
  List<Polynomial> result;
  for (int i=0;i<n;i++) { 
    temp = read_poly(inFile,input_type,poly_type,slp_type,readslp); 
    result.append(temp); 
  }
  return result; 
}

Array<Polynomial> 
read_polyarray(ifstream& inFile, int n, int input_type = SMALLINT, 
  int poly_type = INTMODP, int slp_type = INTMODP, bool readslp=true)
{
  Array<Polynomial> result = Array<Polynomial>(n); 
  for (int i=0;i<n;i++) 
    result[i] = read_poly(inFile,input_type,poly_type,slp_type,readslp); 
  return result; 
}
