
/*************************************************************************

  APU R 2.0

  slp_arithop.c

  (c) 1996, Regents of the University of California. All rights reserved. 

  Author : Ashutosh Rege (rege@cs.berkeley.edu)

*************************************************************************/
  
#include "field.h"
#include "slp.h"
#include "polynomial.h"

const char* Slp::Class_name = "Slp";
int Slp::Eval_level = 1;
int Slp::Degree_eval_level = 1;
extern int Debug; 

/*
Slp 
slp_binop(int op, const Slp& lchild, const Slp& rchild) {
  Slp result = Slp(op,lchild,rchild); 
  return(result);
}
*/


Slp
operator + (const Slp& a, const Slp& b) {
  if (is_zero(a)) return(b); 
  else if (is_zero(b)) return(a);
  else return(slp_binop(ADD,a,b));
}

Slp
operator - (const Slp& a, const Slp&b) {
  if (is_zero(a)) return(-b); 
  else if (is_zero(b)) return(a);
  return(slp_binop(SUB,a,b));
}

Slp
operator - (const Slp& a) {
  if (is_zero(a)) return(Slp::zero_slp); 
  else return(slp_binop(MULT,Slp(-1),a));
}

Slp
operator * (const Slp& a, const Slp&b) {
  if (is_zero(a) || is_zero(b)) return(Slp::zero_slp); 
  else return(slp_binop(MULT,a,b));
}

Slp
operator / (const Slp& a, const Slp& b) {
  if (is_zero(a)) return(Slp::zero_slp);
  else if (is_zero(b)) 
    cout << "Divide by zero_slp in / (Slp,Slp) \n";
  else return(slp_binop(DIV,a,b));
}

Slp
operator ^ (const Slp& a, int b) {
  if (b == 0) 
    return(Slp(1)); 
  else if (b == 1) 
    return(a);
  else {
    return(slp_binop(EXP,a,Slp(b)));
  }
}

int
sign(const Slp& slp) 
{
  if (is_zero(slp)) 
    return(0); 
  else if (op(slp) == NOP_FE) 
    return(sign(value(slp))); 
  else { 
    Polynomial temp = 
      //interpolate_int(slp,name(head_of(Polynomial::var_list)->item_of()),"",false); 
      interpolate_int(slp,"u1","",false); 
    if (degree(temp) > 0) {
      cout << "sign(Slp) called with non-numerical Slp\n";
      return(0); 
    }
    else {
      if (!is_terminal(lcoeff(temp))) {
        cout << "sign(Slp) called with non-numerical Slp\n";
        return(0); 
      }
      else {
	Field_element fe = fe_of(lcoeff(temp)); 
	return(sign(fe)); 
      }
    }
  }
}
