
/*************************************************************************

  APU R 2.0

  slp_det.c

  (c) 1996, Regents of the University of California. All rights reserved. 

  Author : Ashutosh Rege (rege@cs.berkeley.edu)

*************************************************************************/
  
#include "field.h"
#include "matrix.h"
#include "slp.h"
#include "intmodp.h"
#include "slp_det.h"

#include "file_io.h"
#include <iostream.h>
#include <fstream.h>
extern int Debug;

Slp
det (const Matrix<Slp>& M) 
{
	      int non_sing; 
	      int temp_val = 2; 
	      Field_element fe; 
	      Field_element fe2 = make_Field_element(temp_val,INTMODP); 
  int m = dim1_of(M); int n = dim2_of(M);

  Matrix<Slp> A = Matrix<Slp>(m,n); // local copy of M; 
      if (Debug) { 
	Variable* tempv;
	tempv = Slp::var_list.find("s"); 
	cout << tempv->value() << "\n";    
	tempv->value(fe2); 
	cout << tempv->value() << "\n";    
      }
  if (Debug) { 
    ofstream ofile;  
    open_file(ofile,"aputempm2"); 
    ofile << "M := matrix([\n"; 
    for (int i=0;i<m;i++) { 
      ofile << "[";
      for (int j=0;j<n;j++) { 
	      fe = evaluate(M[i][j],"",&non_sing,"s",fe2); 
	      if (j == n-1) ofile << fe << "]"; 
	      else ofile << fe << ","; 
      }
      if (i<n-1) ofile << ",\n"; 
    }  
    ofile << "];"; 

    ofile.close(); 
  }
  for (int i=0;i<m;i++) 
    for (int j=0;j<n;j++) 
      A[i][j] = M[i][j]; 
  int sign = 1;
  Slp divisor = Slp(1);
  int r = 0; 
  for (int k=0;k<n;k++) {
    if (r < m) {
      int p = r; 
      while ((p<m) && (is_zero(evaluate_modp(A[p][k])))) { // Find non-zero pivot.
	p++; 
      }
      if (p < m) {
        if  (p != r) { // Pivot is in row p, so switch rows p and r.
	  for (int j = k;j<n;j++) {
	    Slp temp = A[p][j];
	    A[p][j] = A[r][j]; 
	    A[r][j] = temp;
	  }
	  sign = -sign; 
        }
	for (int i=r+1;i<m;i++) {
	  for (int j=k+1;j<n;j++) {
	    if (Debug) { 
	      fe = evaluate_modp(A[i][j],"",&non_sing,"s",fe2); 
	      cout << fe << "\n"; 
	      fe = evaluate_modp(A[r][k],"",&non_sing,"s",fe2); 
	      cout << fe << "\n"; 
	      fe = evaluate_modp(A[r][j],"",&non_sing,"s",fe2); 
	      cout << fe << "\n"; 
	      fe = evaluate_modp(A[i][k],"",&non_sing,"s",fe2); 
	      cout << fe << "\n"; 
	      fe = evaluate_modp(divisor,"",&non_sing,"s",fe2); 
	      cout << fe << "\n"; 
	    }
	    A[i][j] = (A[r][k]*A[i][j] - A[r][j]*A[i][k])/divisor;
	    if (Debug) { 
	      fe = evaluate_modp(A[i][j],"",&non_sing,"s",fe2); 
	      cout << fe << "\n"; 
	    }
	  }
	  A[i][k] = Slp::zero_slp;
	}
        divisor = A[r][k]; r++;
	if (k == 130) 
	  cout << "130 reached\n"; 
	if (Debug) { 
	  for (int temp_i=135;temp_i<n;temp_i++) { 
	    fe = evaluate_modp(A[136][temp_i],"",&non_sing,"s",fe2); 
	    cout << fe << "\n"; 
	  }
	}
	if (Debug) { 
	  for (int temp_i=135;temp_i<n;temp_i++) { 
	    fe = evaluate_modp(M[136][temp_i],"",&non_sing,"s",fe2); 
	    cout << fe << "\n"; 
	  }
	}
	if (Debug) { 
	  if (Debug) { 
	    fe = evaluate_modp(A[140][140],"",&non_sing,"s",fe2); 
	    cout << fe << "\n"; 
	    fe = evaluate_modp(A[140][141],"",&non_sing,"s",fe2); 
	    cout << fe << "\n"; 
	  }
	  // Slp::Eval_level++; 
	  fe = evaluate_modp(divisor,"",&non_sing,"s",fe2); 
	  cout << fe << "\n"; 
	  while (!non_sing) { 
	    temp_val++; 
	    Slp::Eval_level++; 
	    fe2 = make_Field_element(temp_val,INTMODP); 
	    fe = evaluate_modp(divisor); 
	    cout << fe << "\n"; 
	  }
	  // Slp::Eval_level++; 
	  if (Debug) { 
	    for (int temp_i=0;temp_i<m;temp_i++) {
	      for (int temp_j=0;temp_j<n;temp_j++) { 
	        fe = evaluate_modp(A[temp_i][temp_j],"",&non_sing,"s",fe2); 
	        cout << fe << "\n"; 
	      }
	    }
	  }
	  if (Debug) { 
	    for (int temp_i=0;temp_i<m;temp_i++) {
	      for (int temp_j=0;temp_j<n;temp_j++) { 
	        fe = evaluate_modp(A[temp_i][temp_j],"",&non_sing,"s",fe2); 
	        cout << fe << "\n"; 
	      }
	    }
	  }
	}
      }
      else { 
	cout << "Couldn't find pivot\n"; 
      }
    }
    else k = n; 
  }
  Slp result;
  if (r==m) result = Slp(sign)*divisor; 
  else result = Slp::zero_slp;
  return result;
}

/* Minor expansion version, use for sparse matrices */
Slp
det2 (const Matrix<Slp>& M) 
{
  int m = dim1_of(M); int n = dim2_of(M);
  if (m != n) { 
    cout << "Error in det2(): unequal matrix dimensions\n"; 
    return Slp::zero_slp; 
  }

  Slp result = Slp::zero_slp; 
  for (int i=0;i<n;i++) { 
     if (!is_zero(M[0][i])) { 
       result = result + M[0][i]*cofactor(M,0,i); 
     }
  }
  return result; 
}

Slp
cofactor(const Matrix<Slp>& M, int i, int j) 
{
  Slp result = det2(minor(M,i,j)); 
  if ((i+j) % 2 == 1)
    result = -result; 
  return result; 
}

Matrix<Slp> 
minor (const Matrix<Slp>& M, int i, int j)
{
  int m = dim1_of(M); int n = dim2_of(M); 
  Matrix<Slp> result = Matrix<Slp>(m-1,n-1); 
  bool skip_r,skip_s; 
  skip_r = false; 
  for (int r=0;r<m-1;r++) { 
    if (r >= i) skip_r = true; 
    skip_s = false; 
    for (int s=0;s<n-1;s++) { 
      if (s >= j) skip_s = true; 
      if (skip_r) { 
	if (skip_s) result[r][s] = M[r+1][s+1]; 
	else result[r][s] = M[r+1][s]; 
      }
      else { 
	if (skip_s) result[r][s] = M[r][s+1];  
	else result[r][s] = M[r][s]; 
      }
    }
  }
  return result; 
}


Matrix<Slp>
gauss_elim (const Matrix<Slp>& M) 
{
  int m = dim1_of(M); int n = dim2_of(M);

  Matrix<Slp> A = Matrix<Slp>(m,n); // local copy of M; 
  for (int i=0;i<m;i++) 
    for (int j=0;j<n;j++) 
      A[i][j] = M[i][j]; 
  if (Debug) 
    cout << A << "\n"; 
  int sign = 1;
  Slp divisor = Slp(1);
  int r = 0; 
  for (int k=0;k<n;k++) {
    if (r < m) {
      int p = r; 
      while ((p<m) && (is_zero(evaluate_modp(A[p][k])))) { 
			// Find non-zero pivot.
	p++; 
      }
      if (p < m) {
        if  (p != r) { // Pivot is in row p, so switch rows p and r.
	  for (int j = k;j<n;j++) {
	    Slp temp = A[p][j];
	    A[p][j] = A[r][j]; 
	    A[r][j] = temp;
	  }
	  sign = -sign; 
        }
	for (int i=r+1;i<m;i++) {
	  for (int j=k+1;j<n;j++) {
	    A[i][j] = (A[r][k]*A[i][j] - A[r][j]*A[i][k])/divisor;
	  }
	  A[i][k] = Slp::zero_slp;
	}
        divisor = A[r][k]; r++;
      }
    }
    else k = n; 
  }
  return A;
}

