
/*************************************************************************

  APU R 2.0

  dfs.c

  (c) 1996, Regents of the University of California. All rights reserved. 

  Author : Ashutosh Rege (rege@cs.berkeley.edu)

*************************************************************************/
  
#include "mixvol.h"
#include "mixvol_edges.h"

extern void ludecomp(double**,int,int,int*,double*);

void MixVolEdges::dfs(int depth, int* mv_ptr, int num_polys, bool insert)
{ 
  int min_i, i, j, nedge; 	/* various indices */  
  int TempCard; 
  int vol; 
  int *CurrCards; int* NextCards; int* TestInds;  

  Array<EdgeArray> CurrEdges, NextEdges; 
  Array<Edge> TempEdges; 
  Edge* TestTuple; 

  CurrEdges = AllEdges[depth]; 
  CurrCards = AllCards[depth]; 
  if (depth < num_polys-1) { 
    NextEdges = AllEdges[depth+1]; 
    NextCards = AllCards[depth+1]; 
  }
  TestTuple = AllTestTuple[depth]; 
  TestInds = AllTestInds[depth]; 

  if (depth == 0) { 
    min_i = -1; 
    for (i=0;i<num_polys;i++) { 
      if (CurrCards[i] < 1) { 
	cout << "Depth = 0:" << i << "'th polytope with no edges\n"; 
	return; 
      }
      if (min_i < 0) min_i = i; 
      else if (CurrCards[i] < CurrCards[min_i]) min_i = i; 
      partial_copy_array(NextEdges[i],CurrEdges[i],CurrCards[i]); 
      NextCards[i] = CurrCards[i]; 
    }
    NextCards[min_i] = -1; 
    Path[0][0] = min_i; 

    nedge = 0; 
    for (nedge=0;nedge<CurrCards[min_i];nedge++) { 
      Path[0][1] = nedge; 
      NextEdges[min_i][0] = CurrEdges[min_i][nedge]; 
      dfs(depth+1,mv_ptr,num_polys,insert); 
    }
  } // End. Depth = 0 

  if (0<depth && depth<num_polys-1) { 
    j = 1; 
    for (i=0; i<num_polys; i++) { 
      if (CurrCards[i] == -1) { 
	TestInds[j] = i; 
	TestTuple[j] = CurrEdges[i][0]; 
	j++; 
      }
    }

    min_i = -1; 
    for (i=0;i<num_polys;i++) { 
      if (CurrCards[i] == -1) { 
	NextEdges[i][0] = CurrEdges[i][0]; 
	NextCards[i] = -1; 
      }
      else { 
	TestInds[0] = i; 
	NextCards[i] = filter(depth,i); 
	if (min_i < 0) min_i = i; 
	else if (NextCards[i] < NextCards[min_i]) min_i = i; 
      }
    }
    if (min_i < 0) { 
      cout << "Error in dfs() : at the bottom\n"; 
      return; 
    }

    if (!NextCards[min_i]) return; 

    TempCard = NextCards[min_i]; 
    TempEdges = Array<Edge>(TempCard); 
    partial_copy_array(TempEdges,NextEdges[min_i],TempCard); 
    NextCards[min_i] = -1; 
    Path[depth][0] = min_i; 

    for (nedge=0;nedge<TempCard;nedge++) { 
      Path[depth][1] = nedge; 
      NextEdges[min_i][0] = TempEdges[nedge]; 
      dfs(depth+1,mv_ptr,num_polys,insert); 
    }
  } // End. 0 < Depth < num_polys - 1; 

  if (depth == num_polys - 1) { 
    min_i = -1; 
    j = 1; 
    for (i=0;i<num_polys;i++) { 
      if (CurrCards[i] > -1) { 
	if (min_i < 0) min_i = i; 
	else { 
	  cout << "Error dfs(): more than one polytope "; 
	  cout << "with more than one edge\n"; 
	  return; 
	}
      }
      else { 
	TestInds[j] = i; 
	TestTuple[j] = CurrEdges[i][0]; 
	j++; 
      }
    } // for i;

    if (min_i < 0) { 
      cout << "Error dfs() bottom: no polytope with > 1 edge\n"; 
      return;
    }
    Path[depth][0] = min_i; 
    TestInds[0] = min_i; 
    for (nedge=0;nedge<CurrCards[min_i]; nedge++) { 
      if (TestInds[0] != min_i) { 
	cout << "Error dfs(): min_i = " << min_i << "TestInds[0] = ";
	cout << TestInds[0] << "\n"; 
	return; 
      }
      Path[depth][1] = nedge; 
      TestTuple[0] = CurrEdges[min_i][nedge]; 
      vol = good_any_tuple(depth); 
      if (vol > 0) { 
	*mv_ptr += vol; 
	if (insert) insert_cell(TestTuple,TestInds,vol,num_polys); 
	cell_count++; 
      }
    } // for nedge; 
  }
}

int MixVolEdges::filter(int depth, int indx) 
{ 
  int i,j,cardinality; 

  Array<Edge> destin, source; 
  destin = AllEdges[depth+1][indx]; 
  source = AllEdges[depth][indx]; 
  if (AllTestInds[depth][0] != indx) { 
    cout << "Error in filter()\n"; 
    return(0); 
  }
  j = 0; 
  cardinality = 0; 
  for (i=0;i<AllCards[depth][indx];i++) { 
    AllTestTuple[depth][0] = source[i]; 
    if (good_any_tuple(depth)) { 
      destin[j++] = source[i]; 
      cardinality++; 
    }
  }
  return cardinality; 
}

int MixVolEdges::good_any_tuple(int depth)
{ 
  int i,ip,j,k,ncols,size,icase,NumCons,volume,Maxm,num; 
  Edge* tuple; 
  int* inds; 
  tuple = AllTestTuple[depth]; 
  inds = AllTestInds[depth]; 
  Maxm = depth; 
  num = depth+1; 

  volume = determinant(tuple,num,dim,simp_mat); 
  if (!volume) return 0; 

  ncols=1; 
  NumCons = dim+Maxm; 
  centroid(tuple,Midpoint,num,dim); 
  for (i=0;i<=Maxm;i++) { 
    ip = inds[i]; 
    size = size_of(sites[ip]); 
    for (k=0;k<size;k++) { 
      ncols++; 
      simp_mat[1][ncols] = (double)(-sites[ip][k][dim-1])/1000.0; 
      for (j=0;j<=Maxm;j++)
	if (i==j) 
	  simp_mat[j+2][ncols] = -1.0; 
        else 
	  simp_mat[j+2][ncols] = 0.0; 
      for (j=0;j<dim-1;j++) 
	simp_mat[j+Maxm+3][ncols] = (double)(-sites[ip][k][j]); 
    }
  }

  for (j=0;j<=Maxm;j++) simp_mat[j+2][1] = 1.0; 
  for (j=0;j<dim-1;j++) simp_mat[j+Maxm+3][1] = Midpoint[j]; 
  simp_mat[1][1] = (double)(Midpoint[dim-1]/1000.0); 

  int izrov[LP_SIZE], iposv[LP_SIZE];
  simplx(simp_mat,NumCons,ncols-1,0,0,NumCons,&icase,izrov,iposv); 
  if (icase < 0) { 
    cout << "good_any_tuple() error \n"; 
    return 0; 
  }
  double eps = 0.0001; 
  return (simp_mat[1][1] <=eps)* volume; 
}

void
centroid (Edge* earr, double* midpt, int length, int dim) 
{ 
  int i, e; 
  for (i=0;i<dim;i++) midpt[i] = 0.0; 
  for (e=0;e<length;e++) 
    for (i=0;i<dim;i++) 
      midpt[i] += (edge_head(earr[e])[i] + edge_tail(earr[e])[i])/2.0; 
}

int
determinant(Edge* earr, int length, int dim, double** mat) 
{ 
  int i,j,e; 
  double d; 
  i = 0; 
  for (e=0;e<length;e++) { 
    i++; 
    for (j=0;j<dim-1;j++) 
      mat[i][j+1] = (double)(edge_head(earr[e])[j] - edge_tail(earr[e])[j]); 
  } 
  if (i!=dim-1) squareup(mat,i,dim-1); 

  int iposv[LP_SIZE]; 
  ludecomp(mat,i,i,iposv,&d); 
  if (d==0.0) return 0;
  for (j=1;j<=i;j++) d *= mat[j][j]; 
  return (int) (fabs(d)+0.5); 
}

void squareup(double** a, int nrows, int ncols)
{ 
  int i,j,k; double sum; 

  for (i=1;i<=nrows;i++) { 
    for (j=i;j<=nrows;j++) { 
      sum = 0.0; 
      for (k=1;k<=ncols;k++) 
	sum += a[i][k] * a[j][k]; 
      a[i-1][j] = sum; 
    }
  } 
  for (i=nrows;i>=1;i--) { 
    for (j=nrows;j>=i;j--) 
      a[i][j] = a[j][i] = a[i-1][j]; 
  }
}

/* Stores the cells computed and the associated mixed volumes for future
   use if required. The cells are stored as arrays of edges, one array
   for each cell. The cells are made up of edges, one from each polytope,
   and the order they are stored in corresponds to the order of the
   polytopes. The cells themselves are ordered lexicographically on
   edge indices. 

   Algorithm details: The array "tuple" contains the edges comprising
   the cell which has non-zero volume "vol". The array "inds" gives
   which edge in "tuple" corresponds to which polytope (ind[i] = polytope
   of tuple[i]), the array tuple is essentially resorted according to 
   the polytope indices and stored in MixedCellEdges. MixedVols stores
   the volume of the given cell. The edges are then resorted (actually
   only the cell indices given in CellIndex are changed) to be
   in lexicographic order. 

*/ 
void 
MixVolEdges::insert_cell(Edge* tuple, int* inds, int vol, int num_polys) 
{ 
  int i,j,a,b; 
  short smaller,same; 

  for (j=0;j<num_polys;j++) 
    pos[inds[j]] = j; 

  for (j=0;j<num_polys;j++) 
    MixedCellEdges[cell_count][j] = tuple[pos[j]]; 

  MixedVols[cell_count] = vol; 
  CellIndex[cell_count] = cell_count; 

  i = cell_count-1; 
  smaller = 1; 
  while (smaller && i>=0) { 
    a = CellIndex[i]; b = CellIndex[i+1]; 
    same = 1; j = 0; 
    while (j < num_polys && same) { 
      same = (index_of(MixedCellEdges[a][j])==index_of(MixedCellEdges[b][j]));
      j++; 
    }
    if (same) { 
      cout << "Error in insert_cell() : Found two identical tuples\n"; 
      return; 
    }
    j--; 
    if (smaller = 
	(index_of(MixedCellEdges[b][j]) < index_of(MixedCellEdges[a][j]))) {

      CellIndex[i] = b; 
      CellIndex[i+1] = a; 
    } 
    i--; 
  }
}

/*
static void 
check_normal (Edge* tuple, int* inds, int num_polys)
{ 
  int i,j,entry,coord; 
  double dd; 
  double* normal; 

  short all_distinct, same_normal; 
  static short init; 
  static double** chk_mat; 
  static double* chk_b; 

  if (!tuple && !inds) { 
  } 

  for (i=0;i<num_polys;i++) 
    pos[inds[i]] = i; 

  if (!init) { 
    chk_mat = new 
  }
}
*/
