
/*************************************************************************

  APU R 2.0

  poly_io.c

  (c) 1996, Regents of the University of California. All rights reserved. 

  Author : Ashutosh Rege (rege@cs.berkeley.edu)
  Based on code written by I. Emiris, in turn based on code in
  Numerical Recipes in C, (c) 1992, Numerical Recipes Software

*************************************************************************/
  
#include "simplex.h"
#include "nrutil.h"
#include <stdio.h>
#include <math.h>		/* for fabs() */

#define EPS 1.0e-6
#define FREEALL free_ivector(l3,1,m);free_ivector(l2,1,m);\
	free_ivector(l1,1,n+1);

/* maximizes objective function
 * 1st col must be non-negative
 */

void simplx(double** a,int m,int n,int m1,int m2,int m3,int* icase,
int izrov[],int iposv[])
{
	short deb = 0;
	int i,ip,ir,is,k,kh,kp,m12,nl1,nl2;
	int *l1,*l2,*l3;
	double q1,bmax;

	if (m != (m1+m2+m3)) nrerror("Bad input constraint counts in SIMPLX");
	l1=ivector(1,n+1);
	l2=ivector(1,m);
	l3=ivector(1,m);
	nl1=n;
	for (k=1;k<=n;k++) l1[k]=izrov[k]=k;
	nl2=m;
	for (i=1;i<=m;i++) {
		if (a[i+1][1] < 0.0) {
		  fprintf(stderr, "Negative entry a[%d][1]=%.1f:",i+1,a[i+1][1]);
		  nrerror("Bad input tableau in SIMPLX");
		}
		l2[i]=i;
		iposv[i]=n+i;
	}
	for (i=1;i<=m2;i++) l3[i]=1;
	ir=0;
	if (m2+m3) {
		ir=1;
		for (k=1;k<=(n+1);k++) {
			q1=0.0;
			for (i=m1+1;i<=m;i++) q1 += a[i+1][k];
			a[m+2][k] = -q1;
		}
		do {
			simp1(a,m+1,l1,nl1,0,&kp,&bmax);
			if (bmax <= EPS && a[m+2][1] < -EPS) {
				*icase = -1;
				FREEALL return;
			} else if (bmax <= EPS && a[m+2][1] <= EPS) {
				m12=m1+m2+1;
				if (m12 <= m) {
					for (ip=m12;ip<=m;ip++) {
						if (iposv[ip] == (ip+n)) {
						  simp1(a,ip,l1, nl1,1,&kp,&bmax);
/* Emiris 6/15: > 0.0: bug, 1/bmax is pivot at simp3 */
/* also 6/26/93: clean up if nonzero (incl. <0) */	if ( fabs(bmax) >= EPS)
								goto one;
						}
					}
				}
				ir=0;
				--m12;
				if (m1+1 <= m12)
					for (i=m1+1;i<=m12;i++)
						if (l3[i-m1] == 1)
							for (k=1;k<=n+1;k++)
								a[i+1][k] = -a[i+1][k];
				break;
			}
			simp2(a,n,l2,nl2,&ip,kp,&q1);
			if (ip == 0) {
				*icase = -1;
				FREEALL return;
			}
one:			simp3(a,m+1,n,ip,kp);
			if (iposv[ip] >= (n+m1+m2+1)) {
				for (k=1;k<=nl1;k++)
					if (l1[k] == kp) break;
				--nl1;
				for (is=k;is<=nl1;is++) l1[is]=l1[is+1];
				a[m+2][kp+1] += 1.0;
				for (i=1;i<=m+2;i++) a[i][kp+1] = -a[i][kp+1];
			} else {
				if (iposv[ip] >= (n+m1+1)) {
					kh=iposv[ip]-m1-n;
					if (l3[kh]) {
						l3[kh]=0;
						a[m+2][kp+1] += 1.0;
						for (i=1;i<=m+2;i++)
							a[i][kp+1] = -a[i][kp+1];
					}
				}
			}
			is=izrov[kp];
			izrov[kp]=iposv[ip];
			iposv[ip]=is;
		} while (ir);
	}
	/* end of phase 1, have feasible sol, now optimize it */
	for (;;) {
		simp1(a,0,l1,nl1,0,&kp,&bmax);
		if (bmax <= /*EPS*/0.0) {
			*icase = 0;
			FREEALL return;
		}
		simp2(a,n,l2,nl2,&ip,kp,&q1);
		if (ip == 0) {
			printf("Unbounded:");
			*icase=1;		/* unbounded */
			FREEALL return;
		}
		simp3(a,m,n,ip,kp);
		is=izrov[kp];
		izrov[kp]=iposv[ip];
		iposv[ip]=is;
	}/*for ;;*/
}



/* JFC 5/17/93 init'tion bug, assumes nll>0, but can be 0, => ll[1] is garbage */
/*	*kp=ll[1]; */
/*	*bmax=a[mm+1][*kp+1]; */
/* Emiris 6/15: init'tion still buggy, if all candidates=0 => kp is garbage */
/*	*bmax = 0.0;			*/
/* Emiris 6/21: in picking max abs value, should favor the positive ones */

void simp1(double** a,int mm,int ll[],int nll,int iabf,int* kp,double* bmax)
{
	int k;
	double test;

	if( nll <= 0) {			/* init'tion: fixed */
		*bmax = 0.0;
		return;
	}
	*kp=ll[1];
	*bmax=a[mm+1][*kp+1];
	for (k=2;k<=nll;k++) {
		if (iabf == 0) {
			test=a[mm+1][ll[k]+1]-(*bmax);
			if (test > 0.0) {
			  *bmax=a[mm+1][ll[k]+1];
			  *kp=ll[k];
			}
		}
		else {			/* abs values: have fixed it */
			test=fabs(a[mm+1][ll[k]+1])-fabs(*bmax);
			if (test > 0.0) {
			  *bmax=a[mm+1][ll[k]+1];
			  *kp=ll[k];
			}
		}
	}
}

void simp2(double** a,int n,int l2[],int nl2,int* ip,int kp,double* q1)
{
	int k,ii,i;
	double qp,q0,q;

	*ip=0;
	for (i=1;i<=nl2;i++) {
		if (a[l2[i]+1][kp+1] < -EPS) {
			*q1 = -a[l2[i]+1][1]/a[l2[i]+1][kp+1];
			*ip=l2[i];
			for (i=i+1;i<=nl2;i++) {
				ii=l2[i];
				if (a[ii+1][kp+1] < -EPS) {
					q = -a[ii+1][1]/a[ii+1][kp+1];
					if (q - *q1 < -EPS) {
						*ip=ii;
						*q1=q;
					} else if (q - *q1 < EPS) {
						for (k=1;k<=n;k++) {
							qp = -a[*ip+1][k+1]/a[*ip+1][kp+1];
							q0 = -a[ii+1][k+1]/a[ii+1][kp+1];
							if (q0 != qp) break;
						}
						if (q0 < qp) *ip=ii;
					}
				}/*if a[ii..][]<-EPS*/
			}/*for i*/
		}/*if a[][]<-EPS*/
	}/*for i*/
}

void simp3(double** a,int i1,int k1,int ip,int kp)
{
  int kk,ii;
  double piv;

  piv=1.0/a[ip+1][kp+1];
  for (ii=1;ii<=i1+1;ii++)
    if (ii-1 != ip) {
      a[ii][kp+1] *= piv;
      for (kk=1;kk<=k1+1;kk++)
        if (kk-1 != kp)
          a[ii][kk] -= a[ip+1][kk]*a[ii][kp+1];
    }
  for (kk=1;kk<=k1+1;kk++)
    if (kk-1 != kp) a[ip+1][kk] *= -piv;
      a[ip+1][kp+1]=piv;
}
