/*************************************************************************

  APU R 2.0

  sparse_mat.c

  (c) 1996, Regents of the University of California. All rights reserved. 

  Author : Ashutosh Rege (rege@cs.berkeley.edu)

*************************************************************************/
  
#include "sparse_mat.h"
extern int Debug; 

Matrix<Slp> 
create_matrix(char* fname_mat, char* fname_ind, int num_polys, 
int max_monoms, int dim, Array<SlpArray> coeffs,bool metis=false,
char* metis_file = "")
{ 
  Array<MonomArray> monom_inds = get_inds(fname_ind,num_polys,max_monoms,dim);  
  Matrix<Slp> result = read_create_matrix(fname_mat,num_polys,max_monoms,
  dim,coeffs,monom_inds,metis,metis_file); 
  if (metis) { 
    system("./ometis aputemp.metis"); 
    result = metis_transform(result,metis_file); 
  }
  return result; 
}

Array<MonomArray>
get_inds(char* fname_ind,int num_polys,int max_monoms,int dim)
{
  ifstream file; 
  open_file(file,fname_ind); 
  if (!file) { 
    cout << "Cannot open " << fname_ind << "\n"; 
    return 0; 
  }
  Array<MonomArray> result = Array<MonomArray>(num_polys); 
  int* count = new int[num_polys]; 
  for (int i=0;i<num_polys;i++) {
    count[i] = 0; 
    result[i] = Array<IntPoint>(max_monoms); 
    for (int j=0;j<max_monoms;j++) 
      result[i][j] = Array<int>(dim);
  }
  int nrows,temp,junk; 
  file >> nrows; 
  for (int i=0;i<nrows;i++) { 
    file >> temp; 
    file >> junk; 
    for (int j=0;j<dim;j++) 
      file >> result[temp][count[temp]][j]; 
    count[temp]++; 
  }
  for (int i=0;i<num_polys;i++)
    result[i].set_size(count[i]); 
  file.close(); 
  delete count; 
  return result; 
}

Matrix<Slp> read_create_matrix(char* fname_mat,int num_polys,int max_monoms,
  int dim,Array<SlpArray> coeffs,Array<MonomArray> monom_inds,
  bool metis =false,char* metis_file="") 
{
  ifstream infile; // File where matrix is stored as o/p from "far". 
  ofstream mfile;  // METIS file 
  int num_edges = 0;   // Num of edges in graph from matrix, reqd for Metis. 
  open_file(infile,fname_mat);
  if (!infile) { 
    cout << "Cannot open " << fname_mat << "\n"; 
    return Matrix<Slp>(0,0); 
  }
  Matrix<int> metis_mat; 

  // Read over leading junk, incl. no. of rows/cols, rows > cols. 
  char word[10]; 
  while (strcmp(word,"this")) 
    infile >> word; 
  infile >> word; infile >> word; 
  int rows,cols;
  infile >> rows; infile >> word; infile >> cols; 

  char* metis_file2; 
  if (metis) { 
    metis_file2 = new char[strlen(metis_file) + 2]; 
    strcpy(metis_file2,metis_file); 
    strcat(metis_file2,"2"); 
    metis_mat = Matrix<int>(cols,cols); 
    for (int temp_i=0;temp_i<cols;temp_i++) 
      for (int temp_j=0;temp_j<cols;temp_j++) 
	metis_mat[temp_i][temp_j] = 0; 
  }

  // Find start of data. 
  while (strcmp(word,"#")) infile >> word; 
  infile >> word; 
  while (strcmp(word,"#")) infile >> word; 

  // Now to read in the matrix, first the monomial index, then the row
  // for each row. 

  Matrix<Slp> result = Matrix<Slp>(cols,cols); 

  int poly_ind, cpoly_ind, cnum; 
  Array<int> cur_monom = Array<int>(dim); 
  int cur_row = 0; 
  for (int i=0;i<rows;i++) { 
    infile >> word; 
    sscanf(word,"f_%d",&poly_ind); 
    poly_ind--; // Because of weird convention shift in Yiannis' code output.
    for (int j=0;j<dim;j++)
      infile >> cur_monom[j];
    infile >> word;
    if (!is_validindex(poly_ind,cur_monom,monom_inds)) { 
      //skip_over_line(infile); 
      for (int j=0;j<cols;j++) { 
	infile >> word; 
      }
      while (strcmp(word,"#") && strcmp(word,"]):")) 
	infile >> word; 
    }
    else { 
      if (strcmp(word,"[")) {
        cout << "Error in read_create_matrix(): matrix file error\n";
        return Matrix<Slp>(0,0);
      }
      for (int j=0;j<cols;j++) { 
        infile >> word; 
        if ((isdigit(word[0])) && (word[0] == '0')) { 
	  result[cur_row][j] = Slp::zero_slp;
        }
        else {
          char junk2[5];
          sscanf(word,"c%dx%d%s",&cpoly_ind,&cnum,junk2);
          cpoly_ind--;
          if (poly_ind != cpoly_ind)
            cout << "Warning in read_create_matrix() - mismatching indices\n";
          cnum--;
	  if (type_of(coeffs[poly_ind][cnum]) != SLP) 
	    result[cur_row][j] = Slp(coeffs[poly_ind][cnum]); 
	  else 
	    result[cur_row][j] = (Slp&)(coeffs[poly_ind][cnum]);  
	  if (metis) { 
	    if ((j!= cur_row) && (metis_mat[j][cur_row] == 0))
	      num_edges++; 
	    if (j != cur_row) { 
	      metis_mat[j][cur_row] = 1; 
	      metis_mat[cur_row][j] = 1; 
	    }
	  } 
        }
      }
      cur_row++; 
      while (strcmp(word,"#") && strcmp(word,"]):")) 
	infile >> word; 
    }
  }
  if (metis) {
    post_process(metis_file,metis_file2,num_edges,metis_mat); 
  }
  return result;
}

bool
is_validindex(int polyind, Array<int> cur_monom, Array<MonomArray> monom_inds)
{
    bool result = false; 
    for (int j=0;j<size_of(monom_inds[polyind]);j++) { 
      if (same_point(monom_inds[polyind][j],cur_monom))
	result = true; 
    }
    return result;
}

void
skip_over_line(ifstream file) 
{ 
  char c; 
  file.get(c); 
  while (c != '\n') { 
      file.get(c); 
      cout << c; 
  }
}

/* Inserts the number of edges into the Metis file as required by Metis. 
   Couldn't do it earlier because we didn't know how many edges there
   were till we processed (and wrote) the entire matrix. */
void
post_process(char* mfile_name, char* mfile_name2, int num_edges, 
Matrix<int> metis_mat) 
{
  ofstream file; 
  fstream file2; 
  open_file(file,mfile_name); 

  bool has_edges; 
  file << dim1_of(metis_mat) << " " << num_edges << "\n"; 
  for (int i=0;i<dim1_of(metis_mat);i++) {
    has_edges = false; 
    for (int j=0;j<dim2_of(metis_mat);j++) { 
      if (metis_mat[i][j] == 1) {
	file << j+1 << " "; 
	has_edges = true; 
      }
    }
    if (has_edges) 
        file << "\n"; 
  }

  file.close(); 
}

/* Reads the output from ometis which is stored in the file 
   "mfile_name.perm"; this is a file of indices one per row, with 
   the ith row containing the new index of the ith vertex; 
   metis_transform then returns a new matrix with the vertices/indices 
   reordered accordingly. */  
Matrix<Slp>
metis_transform(Matrix<Slp> in_mat, char* mfile_name) 
{
  char* permfile_name = new char[strlen(mfile_name) + 6]; 
  strcpy(permfile_name,mfile_name);
  strcat(permfile_name,".perm"); 
  ifstream infile; 
  open_file(infile,permfile_name); 
  int rows = dim1_of(in_mat); 
  int cols = dim2_of(in_mat); 
  if (rows != cols) { 
    cout << "Number of row != number of cols in metis_transform()\n"; 
    return Matrix<Slp>(0,0); 
  }

  int new_rownum; 
  Array<int> new_inds = Array<int>(rows); 
  for (int i=0;i<rows;i++) { 
     infile >> new_inds[i]; 
     new_inds[i] = new_inds[i]-1; // Because metis' indices start from 1, 
				  // ours from 0. 
  }
  Matrix<Slp> result = Matrix<Slp>(rows,cols);
  for (int i=0;i<rows;i++) { 
    for (int j=0;j<cols;j++) { 
      result[new_inds[i]][new_inds[j]] = in_mat[i][j]; 
    }
  }
  if (Debug) { 
    for (int i=0;i<rows;i++) { 
      for (int j=0;j<cols;j++) { 
	if (is_zero(in_mat[i][j])) 
	  cout << "0 "; 
	else cout << "1 "; 
      }
      cout << "\n"; 
    }
    for (int i=0;i<rows;i++) { 
      for (int j=0;j<cols;j++) { 
	if (is_zero(result[i][j])) 
	  cout << "0 "; 
	else cout << "1 "; 
      }
      cout << "\n"; 
    }
  }
  return result; 
}

Array<int> 
get_order(Array<int> arr) 
{
  Array<int> result = Array<int>(size_of(arr)); 
  int cur_min = 2*size_of(arr) + 5; 
  int sorted_to = -1; 
  int cur_min_index = -1; 
  for (int i=0;i<size_of(arr);i++) {
    for (int j=0;j<size_of(arr);j++) {
      if ((arr[j] > sorted_to) && (arr[j] < cur_min)) {
	cur_min = arr[j]; 
	cur_min_index = j; 
      }
    }
    sorted_to = cur_min; 
    result[i] = cur_min_index; 
    cur_min = 2*size_of(arr) + 5; 
  }
  return result; 

}

Matrix<Slp> 
elim_zero_rowcol(Matrix<Slp> inmat)
{
  int m = dim1_of(inmat); 
  int n = dim2_of(inmat); 
  int zero_rows = 0; int zero_cols = 0; 
  Array<int> row_count = Array<int>(m); 
  Array<int> col_count = Array<int>(n); 
  for (int i=0;i<m;i++) { 
    if (is_zerorow(inmat[i])) { 
      row_count[i] = 1; 
      zero_rows++; 
    }
    else row_count[i] = 0; 
  }
  for (int j=0;j<n;j++) { 
    if (is_zerocol(inmat,j)) { 
      col_count[j] = 1; 
      zero_cols++; 
    }
    else col_count[j] = 0; 
  }
  int new_m = m - zero_rows; int new_n = n - zero_cols; 
  Matrix<Slp> result = Matrix<Slp>(new_m,new_n); 
  int cur_i = 0; int cur_j = 0; 
  for (int i=0;i<new_m;i++) { 
    while (row_count[cur_i])
      cur_i++; 
    cur_j = 0; 
    for (int j=0;j<new_n;j++) { 
      while (col_count[cur_j]) 
	  cur_j++; 
      result[i][j] = inmat[cur_i][cur_j]; 
      cur_j++; 
    }
    cur_i++; 
  }
  return result; 
}

bool
is_zerorow(Array<Slp> row) 
{ 
  for (int i=0;i<size_of(row);i++) 
    if (!is_zero(row[i]))
      return false; 
  return true; 
}

bool
is_zerocol(Matrix<Slp> mat, int col)
{ 
  for (int i=0;i<dim1_of(mat);i++) 
    if (!is_zero(mat[i][col]))
      return false; 
  return true; 
}

